classdef CHPSystem < System
    
    properties
        Pump
        Collector
        Expander
        Condenser
        Generator
        FluidName
        TrackingAxes
        EtaThermaltoEl
        WpEl % electrical rated power per m^2 collector
        WpTh % thermal rated power per m^2 collector
    end
    
    methods
        function sys = CHPSystem(pump, collector, expander, condenser, generator, fluidName, trackingAxes)
            if nargin == 0
                warning('Null input constructor call: CHPSystem');
            elseif nargin == 7
                if ~isa(pump,'Pump')
                    error('Please specify a Pump object as the 1st input')
                else
                    sys.Pump = pump;
                end
                
                if ~(isa(collector,'ThermalCollector') || isa(collector,'PVTCollector'))
                    error('Please specify a ThermalCollector object as the 2nd input')
                else
                    sys.Collector = collector;
                end
                
                if ~isa(expander,'Expander')
                    error('Please specify a Turbine object as the 3rd input')
                else
                    sys.Expander = expander;
                end
                
                if ~isa(condenser,'Condenser')
                    error('Please specify a Condenser object as the 4th input')
                else
                    sys.Condenser = condenser;
                end
                
                if ~isa(generator,'Generator')
                    error('Please specify a Generator object as the 5th input')
                else
                    sys.Generator = generator;
                end
                
                if ~ischar(fluidName)
                    error('Please specify a fluid name as the 6th input')
                else
                    sys.FluidName = fluidName;
                end
                
                if ~((trackingAxes>=0)&&(trackingAxes<=2))
                    error('Please specify number of tracking axes between 0 (no-tracking) and 2 (2-axis tracking) as the 7th input')
                else
                    sys.TrackingAxes = trackingAxes;
                end
            else
                error('Not correct number of inputs. Please specify 7 inputs for a CHPSystem')
            end
            
        end %end constructor
        
        % the rated power (Wp) is calculated as the collector rated power
        % times the Rankine cycle efficiency and generator efficiency.  The
        % EtaRankine property is also set.
        function calculateWp(sys,etaRankine)
            sys.WpEl = sys.Collector.Wp * etaRankine * sys.Generator.Eta;
            sys.WpTh = sys.Collector.Wp * (1 - etaRankine) * sys.Condenser.Eta;
            sys.EtaThermaltoEl = etaRankine * sys.Generator.Eta;
        end
        
    end % end methods block
    
    methods (Access = public)
        [heatIn, heatOut, etaTherm, fractionCarnotElec] = calculate(sys, env, Thigh, Phigh, Plow);
    end
end