classdef CPVCollector < handle
    %CPVCOLLECTOR Concentrating PV collector class
    %   CPV collectors are implemented as one specific empirical model from
    %   a Semprius collector developed by Sandia.
    
    properties
        Wp % Wp per m^2 module area
        Area % module area
    end
    
    methods
        function collector = CPVCollector(wp, area)
            if nargin == 0
                warning('Null input constructor call: CPVCollector');
            elseif nargin == 2
                collector.Wp = wp;    
                collector.Area = area;
            else
                error('Not correct number of inputs. Please specify 1 input for a CPVCollector')
            end
        end
        
        
        function etaPV = getEfficiency(collector, G, Ta, Ws, pressure, zenith)
            % Calculate efficiency of Semprius HCPV module from Sandia memo
            % Performance model for Semprius module RDD-MOD-296, PSEL 2941
            k = 1.38e-23;
            q = 1.6e-19;
            
            E0 = 1000;
            T0 = 25;
            a = -3.6347;
            b = -0.0830;
            deltaT = 20.95;
            Ns = 30;
            
            % Calculate airmass from Fritz Kasten and Andrew Young. "Revised optical air mass tables and
            %   approximation formula". Applied Optics 28:47354738
            
            % Airmass from this equation doesn't compute with zenith values
            % over 90, so set those values to 90.
            zenith(zenith > 90) = 90;
            
            AM = (1./(cosd(zenith)+0.50572.*((6.07995+(90-zenith)).^-1.6364))).*pressure/1013.25;
            
            
            % Coefficients are different if AM is greater than or less than
            % 1.8, so initialize each variable to a ones vector that you 
            % then multiply by coefficients for only the appropriate
            % indexes (either indexLess or indexGreater), depending on if 
            % airmass is less or greater than 1.8.

            a0 = ones(size(G));
            a1 = ones(size(G));
            alphaImp = ones(size(G));
            betaVmp = ones(size(G));
            Imp0 = ones(size(G));
            Vmp0 = ones(size(G));
            C0 = ones(size(G));
            C1 = ones(size(G));
            C2 = ones(size(G));
            C3 = ones(size(G));
            C8 = ones(size(G));
            n = ones(size(G));
            
            
            indexLess = AM <= 1.8;
            indexGreater = AM > 1.8;
            
            %             if AM <= 1.8
            
            a0(indexLess) = 0.892.*a0(indexLess);
            a1(indexLess) = 0.0725.*a1(indexLess);
            %                 alphaIsc = 4.83e-4;
            alphaImp(indexLess) = 2.32e-4.*alphaImp(indexLess);
            betaVmp(indexLess) = -0.1472.*betaVmp(indexLess);
            %                 betaVoc = -0.1290;
            %                 Isc0 = 1.0172;
            Imp0(indexLess) = 0.9947.*Imp0(indexLess);
            Vmp0(indexLess) = 90.916.*Vmp0(indexLess);
            %                 Voc0 = 103.00;
            C0(indexLess) = 0.9196.*C0(indexLess);
            C1(indexLess) = 0.0804.*C1(indexLess);
            C2(indexLess) = 1.7636.*C2(indexLess);
            C3(indexLess) = -28.477.*C3(indexLess);
            C8(indexLess) = 0.*C8(indexLess);
            n(indexLess) = 4.383.*n(indexLess);
            
            %             elseif AM > 1.8
            a0(indexGreater) = 1.14.*a0(indexGreater);
            a1(indexGreater) = -0.067.*a1(indexGreater);
            %                 alphaIsc = 7.77e-4;
            alphaImp(indexGreater) = 2.32e-4.*alphaImp(indexGreater);
            betaVmp(indexGreater) = -0.1472.*betaVmp(indexGreater);
            %                 betaVoc = -0.1290;
            %                 Isc0 = 1.0172;
            Imp0(indexGreater) = 0.9944.*Imp0(indexGreater);
            Vmp0(indexGreater) = 88.555.*Vmp0(indexGreater);
            %                 Voc0 = 103.02;
            C0(indexGreater) = 0.9892.*C0(indexGreater);
            C1(indexGreater) = 0.0108.*C1(indexGreater);
            C2(indexGreater) = -3.7087.*C2(indexGreater);
            C3(indexGreater) = -33.0179.*C3(indexGreater);
            C8(indexGreater) = -85.7881.*C8(indexGreater);
            n(indexGreater) = 4.299.*n(indexGreater);
            
            
            f1 = a0 + a1.*AM;
            
            % for high airmasses (above about 17) f1 becomes negative for this
            % empirical model, so just set f1 to zero in those cases.
            f1(f1 < 0) = 0;
            Ee = f1.*G./E0;
            Tm = G.*exp(a + b.*Ws) + Ta;
            Tc = Tm + G./E0.*deltaT;
            delta = n.*k.*(Tc + 273.15)./q;
            Vmp = Vmp0 + C2.*Ns.*delta.*log(Ee) + C3.*Ns.*(delta.*log(Ee)).^2 + C8.*Ns.*(delta.*log(Ee)).^3 + betaVmp.*(Tc - T0);
            
            % Any complex or NaN values that may result when Ee is zero are
            % set to zero.
            Vmp(Ee == 0) = 0;
            Imp = Imp0.*(C0.*Ee + C1.*Ee.^2).*(1 + alphaImp.*(Tc-T0));
            
            etaPV = zeros(size(G));
            etaPV(G>0) = (Imp(G>0).*Vmp(G>0))/collector.Area./G(G>0);
        end
        
    end
    
end

