classdef FluidState < handle
    % Calculates fluid state of any fluid using RefProp
    % Units
    % Density: kg/m^3
    % Specific enthalpy: kJ/kg
    % Specific entropy: kJ/(kg K)
    % Specific internal energy: kJ/kg
    % Pressure: kPa
    % Quality: 0-1 (liquid->vapor)
    % Temperature: K


    
    properties
        FluidModel
        Density
        Enthalpy
        Entropy
        InternalEnergy
        Pressure
        Quality
        Temperature
    end
    
    methods
        function state = FluidState(fluidModel, prop1name, prop1, prop2name, prop2)
            state.FluidModel= fluidModel;
            % fluid and two properties are specified
            if nargin == 5
                % remove whitespace from beginning and end of property
                % names and make lowercase and put into cell array with the
                % property value
                property1 = {lower(strtrim(prop1name)) prop1};
                property2 = {lower(strtrim(prop2name)) prop2};
                
                % Alphabetize property pairs and reassign the first
                % alphabetically to property1 if not already
                sortedPropNames = sort({property1{1} property2{1}});
                if strcmp(sortedPropNames{1}, property2{1})
                    tempVar = property1;
                    property1 = property2;
                    property2 = tempVar;
                end
                
                switch property1{1}
                    case 'enthalpy'
                        state.Enthalpy = property1{2};
                        switch property2{1}
                            case 'entropy'
                                state.Entropy = property2{2};
                                [state.InternalEnergy state.Pressure state.Quality state.Temperature state.Density] = refpropm('UPQTD','H',state.Enthalpy,'S',state.Entropy,state.FluidModel);
%                                 state.Pressure = refpropm('P','H',state.Enthalpy,'S',state.Entropy);
%                                 getStatePT();
                            case 'internal energy'
                                error('Must input an independent property other than %s', property2{1});
                            case 'pressure'
                                state.Pressure = property2{2};
                                [state.Entropy state.InternalEnergy state.Quality state.Temperature state.Density] = refpropm('SUQTD','P',state.Pressure,'H',state.Enthalpy,state.FluidModel);
%                                 getStatePT();
                            case 'quality'
                                state.Quality = property2{2};
                                error('Must input an independent property other than %s', property2{1});
                            case 'temperature'
                                state.Temperature = property2{2};
                                error('Must input an independent property other than %s', property2{1});
                            otherwise
                                error('Invalid Property Name: %s', property2{1});
                        end
                    case 'entropy'
                        state.Entropy = property1{2};
                        switch property2{1}
                            case 'internal energy'
                                state.InternalEnergy = property2{2};
                                error('Must input an independent property other than %s', property2{1});
                            case 'pressure'
                                state.Pressure = property2{2};
                                [state.Enthalpy state.InternalEnergy state.Quality state.Temperature state.Density] = refpropm('HUQTD','P',state.Pressure,'S',state.Entropy,state.FluidModel);
%                                 state.Enthalpy = refpropm('H','P',state.Pressure,'S',state.Entropy);
%                                 getStatePT();
                            case 'quality'
                                state.Quality = property2{2};
                                error('Must input an independent property other than %s', property2{1});
                            case 'temperature'
                                state.Temperature = property2{2};
                                error('p(T,s) = no go')
                            otherwise
                                error('Invalid Property Name: %s', property2{1});
                        end
                    case 'internal energy'
                        state.InternalEnergy = property1{2};
                        error('Must input an independent property other than %s', property2{1});
                    case 'pressure'
                        state.Pressure = property1{2};
                        switch property2{1}
                            case 'quality'
                                state.Quality = property2{2};
                                [state.Enthalpy state.Entropy state.InternalEnergy state.Temperature state.Density] = refpropm('HSUTD','P',state.Pressure,'Q',state.Quality,state.FluidModel);
%                                 state.Temperature = refpropm('T','P',state.Pressure,'H',state.Enthalpy);
%                                 getStatePT();
                                warndlg('Note: Pressure and quality give saturation conditions and do not work for superheat or supercool')
                                
                            case 'temperature'
                                state.Temperature = property2{2};
                                [state.Enthalpy state.Entropy state.InternalEnergy state.Quality state.Density] = refpropm('HSUQD','T',state.Temperature,'P',state.Pressure,state.FluidModel);

%                                 getStatePT();
                            otherwise
                                error('Invalid Property Name: %s', property2{1});
                        end
                    case 'quality'
                        state.Quality = property1{2};
                        switch property2{1}
                            case 'temperature'
                                state.Temperature = property2{2};
                                error('Must input an independent property other than %s', property2{1});
%                                 getStatePT();
                            otherwise
                                error('Invalid Property Name: %s', property2{1});
                        end
                    otherwise
                        error('Invalid Property Name: %s', property1{1});
                end
                % fluid, one property (T or P), and phase are specified at
                % saturation
            elseif nargin == 4
                % remove whitespace from beginning and end of property
                % names and make lowercase and put into cell array with the
                % property value
                property = {lower(strtrim(prop1name)) prop1};
                phase = lower(strtrim(prop2name));
                switch property{1}
                    case 'pressure'
                        state.Pressure = property{2};
                        switch phase
                            case 'saturated liquid'
                                state.Quality = 0;
%                                 state.Enthalpy = refpropm('H','P',state.Pressure,'Q',state.Quality);
%                                 state.Entropy = refpropm('S','P',state.Pressure,'Q',state.Quality);
%                                 state.InternalEnergy = refpropm('U','P',state.Pressure,'Q',state.Quality);
                            case 'saturated vapor'
                                state.Quality = 1;
%                                 state.Temperature = refpropm('T','P',state.Pressure,'Q',state.Quality);
%                                 state.Enthalpy = refpropm('H','P',state.Pressure,'Q',state.Quality);
%                                 state.Entropy = refpropm('S','P',state.Pressure,'Q',state.Quality);
%                                 state.InternalEnergy = refpropm('U','P',state.Pressure,'Q',state.Quality);
                        end
                        [state.Enthalpy state.Entropy state.InternalEnergy state.Temperature state.Density] = refpropm('HSUTD','P',state.Pressure,'Q',state.Quality,state.FluidModel);
                                                        
                    case 'temperature'
                        state.Temperature = property{2};
                        switch phase
                            case 'saturated liquid'
                                state.Quality = 0;
%                                 state.Enthalpy = refpropm('H',state.Pressure);
%                                 state.Entropy = XSteam('sL_p',state.Pressure);
%                                 state.InternalEnergy = XSteam('uL_p',state.Pressure);
                            case 'saturated vapor'
%                                 state.Pressure = XSteam('psat_T',state.Temperature);
%                                 state.Enthalpy = XSteam('hV_p',state.Pressure);
                                state.Quality = 1;
%                                 QualityCheck = XSteam('x_ph',state.Pressure,state.Enthalpy)
%                                 state.Entropy = XSteam('sV_p',state.Pressure);
%                                 state.InternalEnergy = XSteam('uV_p',state.Pressure);
                        end
                        [state.Enthalpy state.Entropy state.InternalEnergy state.Pressure state.Density] = refpropm('HSUPD','T',state.Temperature,'Q',state.Quality,state.FluidModel);

                    otherwise
                        error('Invalid Saturation State Specified %s', property{1});
                end
            else error('invalid number of arguments in function call: %d', nargin);
            end
            
%             function getStatePT()
%                 if isempty(state.Enthalpy)
%                     state.Enthalpy = XSteam('h_pT',state.Pressure,state.Temperature);
%                 end
%                 if isempty(state.Entropy)
%                     state.Entropy = XSteam('s_ph',state.Pressure,state.Enthalpy);
%                 end
%                 if isempty(state.InternalEnergy)
%                     state.InternalEnergy = XSteam('u_ph',state.Pressure,state.Enthalpy);
%                 end
%                 if isempty(state.Quality)
%                     state.Quality = XSteam('x_ph',state.Pressure,state.Enthalpy);
%                 end
%                 if isempty(state.Density)
%                     state.Density = XSteam('rho_ph',state.Pressure,state.Enthalpy);
%                 end
%             end
        end
    end
end

