function pvrad = pvperformance(G,iamPV,incD)

% Calculates effective radiation for the PV reference case, after factoring
% in the PV panel's incidence angle modifier (IAM).
%
% UNITS
% Output is watts per meter squared; input is the same, IAM factors between
% zero and one, and angles in degrees.
%
% VECTORIZATION
% All inputs and outputs are vectorized.  'iamPV' is a vector of IAM values
% at 10 angles (0, 10, ... , 90 degrees).

% Use the direct incidence angle to determine which incidence angle
% modifier value to select.  IAM values are associated with 0, 10, 20, ...
% , 90 degrees.  So, for, example, if incidence angle incD = -32, the
% appropriate incidence angle modifier is the one associated with 30
% degrees--i.e., the 4th entry.  So d = 4. 
d = round(abs(incD)/10) + 1;

% The IAM entries end at 90 degrees, or the 10th entry (where they are
% zero).  Thus the maximum entry number to look up is 10.
d(gt(d,10)) = 10;

% Now, the IAM entry number is used to look up the appropriate IAM value.
% For instance, entry number 10 refers to an IAM value ('Kd') of 0.
% To do this with vector math, it is necessary to expand the 'd' vector
% (which contains values ranging from 1 to 10) to a boolean matrix 'db'
% with the same number of rows, and 10 columns.  Where 'd' equals 4, for
% instance, the 4th column of 'lb' will be 1 and all other columns will be
% zero.

% First the boolean matrix is preallocated.
db = zeros(length(d), 10);

% Second, the boolean matrix is filled one column at a time.
for ref = 1:10
   db(:,ref) = d == ref;
end

% Finally, vector multiplication is used to find the correct IAM values
% (specified in 'iamD'--note that the transpose of the 1x10 vector is used)
% for each time step, resulting in 'Kd'.  For example, wherever the 4th
% column in the boolean matrix 'lb' is true, 'Kl' will be equal to the 4th
% entry in 'iamL'.
Kd = db * iamPV';

% The effective radiation values are now simply the appropriate IAM values
% times the total incident radiation (G).
pvrad = Kd .* G;
