function [Thermalefficiency, PVefficiency] = PVTcalc(~, Tamb, Thtfin, Thtfout, C, G)
% Calculating the PV efficiency and thermal efficiency for the
% PV(C-Si)/Thermal systems.  Outputs are Thermal and PV collector efficiency, but function does not currently return calculated output m, which is mass
% flow rate.

% Calculating the PV efficiency and thermal efficiency for the PV(C-Si)/Thermal systems.

% INPUTS:

% % Ambient Temperature [K]
% Tamb=273.15;
% % Fluid  Temperature In [K]
% Thtfin=372.8428;
% % Fluid  Temperature Out [K]
% Thtfout=500;
% %Concentration Ratio [-]
% C=100;
% % Solar Flux [W/m^2]
% G=418;

% SOLAR CELL Efficiency PARAMETERS:

%Reference Temperature [K]
Tref=298;
%Module's Electrical Efficiency at Reference Temperature [-]
Eref=0.15;
%Temperature Coefficient [1/K]
Bref=0.0041;

%Thermal/Radiative Properties:

%Wind Heat Transfer Coefficient [[W/( m^2 *K)]
hwind=10;
%Emissivity of Glass 3 [ ]
eg3=0.9;
%Stefan-Boltzmann Constant [W/(m^2*K^4)]
s=5.6704E-08;
%Thermal Conductivity of Glass [W/(m*K)]
Kg=0.8;
%Thickness of Glass 3 [m]
dg3=0.003;
%Heat Transfer Coefficient Between Glass 2 and 3 [[W/( m^2 *K)]
h=0;
%Thickness of Glass 2 [m]
dg2=0.003;
%Nusselt Number [ ]
Nu=8.23;
%Thickness of Heat Transfer Fluid [m]
dhtf=0.2;
%Tranmissivity of Heat Transfer Fluid [ ], set up for a realistic absorbing
%fluid for a silicon cell
tf=0.407;
%Absorptivity of Heat Transfer Fluid [ ]
ahtf=1-tf;
%Emissivity of Glass [ ]
eg=0.9;
%Emissivity of P.V. [ ]
ep=0.35;
%Tranmissivity of Glass 2 [ ]
tg2=0.9;
%Tranmissivity of Glass 3 [ ]
tg3=0.9;
%Heat Transfer Coefficient Between P.V. and Glass 1 [[W/( m^2 *K)]
h1pv=0;
%Thickness of Glass 1 [m]
dg1=0.003;
%Tranmissivity of system [ ]
tsys=tg2*tg3*tg3;
%Absorptivity of P.V. [ ]
apv=0.8;
%Heat Transfer Coefficient Between air and P.V. [[W/( m^2 *K)] on the
%backside of the system
hins=1000;
%Thickness Between Glass 2 and 3 [m]:
% dg23=0.025;

% Define constants for variable thermal conductivity and specific heat
% capacity
Ak=-0.0000819477;
Bk=-0.000000192257;
Ck=0.000000000025034;
Dk=-0.0000000000000072974;
Ek=0.137743;
Ac=0.002414;
Bc=0.0000059591;
Cc=-0.000000029879;
Dc=0.000000000044172;
Ec=1.498;



if (G<=0)
    Thermalefficiency = 0;
    PVefficiency = 0;
    m = 0;
    
else
    N=100;
    epsilon=0.05;
    maxval=10000;
    
    % And calculate some often used expressions that don't change on successive
    % iterations
    Tave=(Thtfout+Thtfin)/2;
    TaveC=Tave-273.15; %Average fluid temperature of input and output in Celsius
    TaveCpolyk = Ak*TaveC+Bk*TaveC^2+Ck*TaveC^3+Dk*TaveC^4+Ek;
    TaveCpolyc = Ac*TaveC+Bc*TaveC^2+Cc*TaveC^3+Dc*TaveC^4+Ec;

    
    %First guess of the temperatures
    x=[300; 300; 300; 300; 1; 300; 300; 300];
    
    %     x=x0;
    
    while (N>0)
        
        Tg3a=x(1,1);
        Tg3b=x(2,1);
        Tg2a=x(3,1);
        Tg2b=x(4,1);
        m=x(5,1);
        Tg1a=x(6,1);
        Tg1b=x(7,1);
        Tpv=x(8,1);
        
        %Defining the Function EBE:
        ebe1=tf*tsys*apv*C*G*(1-(Eref*(1-Bref*(Tpv-Tref))))-1/(1/eg+1/ep-1)*s*(Tpv^4-Tg1b^4)-h1pv*(Tpv-Tg1b)-hins*(Tpv-Tamb);
        ebe2=1/(1/eg+1/ep-1)*s*(Tpv^4-Tg1b^4)+h1pv*(Tpv-Tg1b)+Kg/dg1*(Tg1a-Tg1b);
        ebe3=Nu*TaveCpolyk/dhtf*(Tg1a-Tave)+1/(1/eg+1/ep-1)*s*(Tpv^4-Tg2a^4)+Kg/dg1*(Tg1a-Tg1b);
        ebe4=Nu*TaveCpolyk/dhtf*(Tg1a-Tave)+tg2*tg3*(1-tf)*C*G+(1-tf)/(1/eg+1/ep-1)*s*(Tpv^4-Tg2a^4)-Nu*TaveCpolyk/dhtf*(Tave-Tg2b)-m*1000*TaveCpolyc*(Thtfout-Thtfin);
        ebe5=Nu*TaveCpolyk/dhtf*(Tave-Tg2b)+(1-ahtf)/(1/eg+1/ep-1)*s*(Tpv^4-Tg2a^4)+Kg/dg2*(Tg2a-Tg2b);
        ebe6=h*(Tg2a-Tg3b)+eg/(2-eg)*s*(Tg2a^4-Tg3b^4)+Kg/dg2*(Tg2a-Tg2b);
        ebe7=h*(Tg2a-Tg3b)+eg/(2-eg)*s*(Tg2a^4-Tg3b^4)+Kg/dg3*(Tg3a-Tg3b);
        ebe8=hwind*(Tg3a-Tamb)+eg3*s*(Tg3a^4-Tamb^4)+Kg/dg3*(Tg3a-Tg3b);
        
        EBE=[ebe1; ebe2; ebe3; ebe4; ebe5; ebe6; ebe7; ebe8];
        
        J = zeros(8,8);
        
        J(1,7)=h1pv + (4*Tg1b^3*s)/(1/eg + 1/ep - 1);
        J(1,8)=Bref*C*Eref*G*apv*tf*tsys - hins - (4*Tpv^3*s)/(1/eg + 1/ep - 1) - h1pv;
        J(2,6)=Kg/dg1;
        J(2,7)=- h1pv - Kg/dg1 - (4*Tg1b^3*s)/(1/eg + 1/ep - 1);
        J(2,8)=h1pv + (4*Tpv^3*s)/(1/eg + 1/ep - 1);
        J(3,3)=-(4*Tg2a^3*s)/(1/eg + 1/ep - 1);
        %         J(3,6)=Kg/dg1 - (Nu*((6046672997316513*Thtfin)/147573952589676412928 + (6046672997316513*Thtfout)/147573952589676412928 + (7263264103176555*(Thtfin/2 + Thtfout/2 - 273)^2)/37778931862957161709568 - (242113991745861*(Thtfin/2 + Thtfout/2 - 273)^3)/9671406556917033397649408 + (4625276745052741*(Thtfin/2 + Thtfout/2 - 273)^4)/633825300114114700748351602688 - 11814381204047307441/73786976294838206464))/dhtf;
        J(3,6)=Kg/dg1 + Nu *TaveCpolyk/dhtf;
        J(3,7)=-Kg/dg1;
        J(3,8)=(4*Tpv^3*s)/(1/eg + 1/ep - 1);
        J(4,3)=(4*Tg2a^3*s*(tf - 1))/(1/eg + 1/ep - 1);
        %         J(4,4)=-(Nu*((6046672997316513*Thtfin)/147573952589676412928 + (6046672997316513*Thtfout)/147573952589676412928 + (7263264103176555*(Thtfin/2 + Thtfout/2 - 273)^2)/37778931862957161709568 - (242113991745861*(Thtfin/2 + Thtfout/2 - 273)^3)/9671406556917033397649408 + (4625276745052741*(Thtfin/2 + Thtfout/2 - 273)^4)/633825300114114700748351602688 - 11814381204047307441/73786976294838206464))/dhtf;
        J(4,4)=Nu *TaveCpolyk/dhtf;
        %         J(4,5)=1000*(Thtfin - Thtfout)*((5566305024241857*Thtfin)/4611686018427387904 + (5566305024241857*Thtfout)/4611686018427387904 + (3517631763508563*(Thtfin/2 + Thtfout/2 - 273)^2)/590295810358705651712 - (1128796705133297*(Thtfin/2 + Thtfout/2 - 273)^3)/37778931862957161709568 + (6835285926914227*(Thtfin/2 + Thtfout/2 - 273)^4)/154742504910672534362390528 + 241818944523010822387/288230376151711744000);
        J(4,5)=-1000*(Thtfout - Thtfin)*TaveCpolyc;
        %         J(4,6)=-(Nu*((6046672997316513*Thtfin)/147573952589676412928 + (6046672997316513*Thtfout)/147573952589676412928 + (7263264103176555*(Thtfin/2 + Thtfout/2 - 273)^2)/37778931862957161709568 - (242113991745861*(Thtfin/2 + Thtfout/2 - 273)^3)/9671406556917033397649408 + (4625276745052741*(Thtfin/2 + Thtfout/2 - 273)^4)/633825300114114700748351602688 - 11814381204047307441/73786976294838206464))/dhtf;
        J(4,6)= Nu *TaveCpolyk/dhtf;
        J(4,8)=-(4*Tpv^3*s*(tf - 1))/(1/eg + 1/ep - 1);
        J(5,3)=Kg/dg2 + (4*Tg2a^3*s*(ahtf - 1))/(1/eg + 1/ep - 1);
        %         J(5,4)=(Nu*((6046672997316513*Thtfin)/147573952589676412928 + (6046672997316513*Thtfout)/147573952589676412928 + (7263264103176555*(Thtfin/2 + Thtfout/2 - 273)^2)/37778931862957161709568 - (242113991745861*(Thtfin/2 + Thtfout/2 - 273)^3)/9671406556917033397649408 + (4625276745052741*(Thtfin/2 + Thtfout/2 - 273)^4)/633825300114114700748351602688 - 11814381204047307441/73786976294838206464))/dhtf - Kg/dg2;
        J(5,4)= - Nu*TaveCpolyk/dhtf- Kg/dg2;
        J(5,8)=-(4*Tpv^3*s*(ahtf - 1))/(1/eg + 1/ep - 1);
        J(6,2)=(4*Tg3b^3*eg*s)/(eg - 2) - h;
        J(6,3)=h + Kg/dg2 - (4*Tg2a^3*eg*s)/(eg - 2);
        J(6,4)=-Kg/dg2;
        J(7,1)=Kg/dg3;
        J(7,2)=(4*Tg3b^3*eg*s)/(eg - 2) - Kg/dg3 - h;
        J(7,3)=h - (4*Tg2a^3*eg*s)/(eg - 2);
        J(8,1)=hwind + Kg/dg3 + 4*Tg3a^3*eg3*s;
        J(8,2)=-Kg/dg3;
        
        if abs(det(J))<epsilon
            error('Jacobian Matrix is Singular-try new x0');
        end;
        
        x = x-J\EBE;  %Matlab says this is a faster way to calculate x-inv(J)*EBE
        
        if abs(EBE)<epsilon
            finalanswer=x;
            PVefficiency=Eref*(1-Bref*(finalanswer(8,1)-Tref));
            Thermalefficiency=(finalanswer(5,1)*1000*TaveCpolyc*(Thtfout-Thtfin))/(C*G);
            if  Thermalefficiency < 0
                Thermalefficiency=0;
            end
            return;
        end;
        
        if abs(EBE)>maxval
            iter=100-N;
            disp(['iterations=',num2str(iter)]);
            error ('solution divergence');
        end;
        
        N=N-1;
    end;
    error('no convergence after 100 iterations.');
end
end
