function PlotLevels(~, titletext, type, ...
    lolim, hilim, data, clabel)

% Plots any data associated with a site as a colored dot in its appropriate
% position in the World.  Creates two figures: one with seasonal
% data, and one with yearly data.  The color limits for both figures are
% the same.
%
% UNITS
% Input 1) A string of text to append to the figures' titles.
% 2) String specifying the type of plot to return: 'lower' for just the
% lower 48 states of the US, 'US' to include Alaska, Hawaii, Guam, and PR, and 'EU' for Europe.
% 3) Lower limit of color scale, or -1 for autoscale.
% 4) Upper limit of color scale, or -1 for autoscale.
% 5) A structure including fields labeled '.lon' and '.lat' containing the
% latitude and longitude of the TMY sites as vectors.
% 6) A structure containing all the seasonal and yearly data as vectors,
% including fields labeled '.year', '.spr', '.smr', '.fal', and '.win'
% 7) The row number of 'datastruct' from which to take the data.  In the
% case of CHP/PV ratios or other system-related data, this is the system
% number you'd like to view.
%
% VECTORIZATION
% Inputs are vectors within appropriately-named structures.


% % If only the lower 48 states are specified:
% if strcmp(type,'lower');
%
%     % Set the appropriate lat/lon limits for the plots.
%     latmin = 24;
%     latmax = 50;
%     lonmin = -129;
%     lonmax = -65;
%
%     % Remove that data which is outside the lat/lon limits.
%     low48 = ones(1,length(longitude));
%     low48(longitude < lonmin) = 0;
%     low48(longitude > lonmax) = 0;
%     low48(latitude < latmin) = 0;
%     low48(latitude > latmax) = 0;
%     longitude(low48 == 0) = [];
%     latitude(low48 == 0) = [];
%     springdata(low48 == 0) = [];
%     summerdata(low48 == 0) = [];
%     falldata(low48 == 0) = [];
%     winterdata(low48 == 0) = [];
%     yeardata(low48 == 0) = [];
%
%     % Set the dots on the plot to be the right size
%     smark = 30;
%     lmark = 100;

% If all 52 states and territories are desired:
if strcmp(type,'US')
    
    % Shift Guam and Alaska's tail to the left of the map.
    data.longitude(data.longitude > 0) = data.longitude(data.longitude > 0) - 360;
    
    % If Europe is desired:
    % elseif strcmp(type,'Europe')
    %     %
    %     % %     % Set the appropriate lat/lon limits for the plots.
    %     % %     latmin = 34;
    %     % %     latmax = 72;
    %     % %     lonmin = -25;
    %     % %     lonmax = 31;
    %     %
    %     %     % Set the dots on the plot to be the right size
    %     %     smark = 2;
    %     %     lmark = 4;
    %     %     refVec = [4 max(data.latitude) min(data.longitude)];
    %     %     levels = 64;
    % else
    %     error('Invalid type of plot specified: please choose "USA", or "Europe"')
end

% Set the appropriate lat/lon limits for the plots based on the data.
latmin = min(data.latitude);
latmax = max(data.latitude);
% latmax = 62.125;
lonmin = min(data.longitude);
% lonmin = -11.125;
lonmax = max(data.longitude);

% If the lower color limit is set at -1 (autoscale), set it as the lowest
% value found in any of the data.  Otherwise set it to the specified value.
if lolim == -1
    CminSeasonal = min([min(data.springdata) min(data.summerdata) min(data.falldata) min(data.winterdata)]);
    CminYearly = min(data.yeardata);
else
    %     CminSeasonal = lolim/4;
    CminSeasonal = lolim;
    CminYearly = lolim;
end

% If the upper color limit is set at -1 (autoscale), set it as the highest
% value found in any of the data.  Otherwise set it to the specified value.
if hilim == -1
    CmaxSeasonal = max([max(data.springdata) max(data.summerdata) max(data.falldata) max(data.winterdata)]);
    CmaxYearly = max(data.yeardata);
else
    %     CmaxSeasonal = hilim/3;
    CmaxSeasonal = hilim;
    CmaxYearly = hilim;
end

% this is still a problem, but there is also problems with creating the map
% if there is just one latitude and longitude point, so the fix below seems
% irrelevant until the bigger problem is fixed.
% % if there is only one site selected then cMin will equal cMax, but to get
% % a valid color scale set cMin to zero in this case.
% if CmaxYearly == CminYearly
%     CminYearly = 0;
% end
% 
% if CmaxSeasonal == CminSeasonal
%     CminSeasonal = 0;
% end

% Create the first figure, for the seasonal data
figSeasonal = figure('NumberTitle','off',...
    'Name',['Seasonal ' titletext]);

% Load data for coasts and borders from Matlab map toolbox
load coast;


if strcmp(type, 'US')
    % Load data for state borders
    states = shaperead('usastatelo', 'UseGeoCoords', true);
    
    % Set the dots on the plot to be the right size
    smark = 5;
    lmark = 30;
    
    % Create the Spring axes
    subplot(221);
    axes11 = axesm('MapProjection', 'eqaconicstd','MapLatLimit',[latmin latmax],...
        'MapLonLimit', [lonmin lonmax],'Frame','on');
    geoshow(axes11, lat, long, 'Color', 'black');
    geoshow(axes11, states, 'FaceColor', 'none');
    % Create scatter
    caxis(axes11,[CminSeasonal CmaxSeasonal]);
    set(axes11,'Position',[0.01 0.49 0.42 0.42]);
    scatterm(data.latitude,data.longitude,smark,data.springdata,'MarkerFaceColor','flat',...
        'Parent',axes11);
    % Create title
    title({'Spring'});
    tightmap;
    
    % Create the Summer axes
    subplot(222);
    axes12 = axesm('MapProjection', 'eqaconicstd','MapLatLimit',[latmin latmax],...
        'MapLonLimit', [lonmin lonmax],'Frame','on');
    geoshow(axes12, lat, long, 'Color', 'black');
    geoshow(axes12, states, 'FaceColor', 'none');
    % Create scatter
    caxis(axes12,[CminSeasonal CmaxSeasonal]);
    set(axes12,'Position',[0.44 0.49 0.42 0.42]);
    scatterm(data.latitude,data.longitude,smark,data.summerdata,'MarkerFaceColor','flat',...
        'Parent',axes12);
    % Create title
    title({'Summer'});
    tightmap;
    
    % Create the Fall axes
    subplot(223);
    axes13 = axesm('MapProjection', 'eqaconicstd','MapLatLimit',[latmin latmax],...
        'MapLonLimit', [lonmin lonmax],'Frame','on');
    geoshow(axes13, lat, long, 'Color', 'black');
    geoshow(axes13, states, 'FaceColor', 'none');
    % Create scatter
    caxis(axes13,[CminSeasonal CmaxSeasonal]);
    set(axes13,'Position',[0.01 0.10 0.42 0.42]);
    scatterm(data.latitude,data.longitude,smark,data.falldata,'MarkerFaceColor','flat',...
        'Parent',axes13);
    % Create title
    title({'Fall'});
    tightmap;
    
    % Create the Winter axes
    subplot(224);
    axes14 = axesm('MapProjection', 'eqaconicstd','MapLatLimit',[latmin latmax],...
        'MapLonLimit', [lonmin lonmax],'Frame','on');
    geoshow(axes14, lat, long, 'Color', 'black');
    geoshow(axes14, states, 'FaceColor', 'none');
    % Create scatter
    caxis(axes14,[CminSeasonal CmaxSeasonal]);
    set(axes14,'Position',[0.44 0.10 0.42 0.42]);
    scatterm(data.latitude,data.longitude,smark,data.winterdata,'MarkerFaceColor','flat',...
        'Parent',axes14);
    % Create title
    title({'Winter'});
    tightmap;
    
    % Create colorbar
    % A colorbar is created in a custom position to the right of all
    % the subplots.  Since the colormap is associated with the current axes
    % (sub4), that axes must be set back to its original size and position
    % after creating the colormap (i.e. adding colormaps resize axes
    % automatically, so the position is saved beforehand and reset)
    %     sub4Pos=get(sub4, 'Position');
    cbarSeasonal = colorbar('peer',axes14,'Position', [0.87 0.03 0.04 0.9]);
    %     set(sub4,'Position',sub4Pos);
    ylabel(cbarSeasonal, clabel);
    
    % Create the second figure, for the yearly data
    figYearly = figure('NumberTitle','off',...
        'Name',['Yearly ' titletext]);
    axes2 = axesm('MapProjection', 'eqaconicstd','MapLatLimit',[latmin latmax],...
        'MapLonLimit', [lonmin lonmax],'Frame','on');
    %     set(axes2,'Parent',figYearly);
    geoshow(axes2, lat, long, 'Color', 'black');
    states = shaperead('usastatelo', 'UseGeoCoords', true);
    geoshow(axes2, states, 'FaceColor', 'none');
    
    % Create scatter
    caxis(axes2,[CminYearly CmaxYearly]);
    set(axes2,'Position',[0.01 0.08 0.85 0.85]);
    scatterm(data.latitude,data.longitude,lmark,data.yeardata,'MarkerFaceColor','flat',...
        'Parent',axes2);
    
    % Create colorbar
    cbar2 = colorbar('peer',axes2,'Position',[0.87 0.03 0.04 0.9]);
    ylabel(cbar2, clabel);
    tightmap
    
elseif strcmp(type, 'EU')
    
    % Create a regular grid of the data so that it can be plotted as a
    % contour plot.
    % First find the size of the array assuming data every quarter degree
    % from the min to max latitude/longitude.
    xLength = length(latmin:0.25:latmax);
    yLength = length(lonmin:0.25:lonmax);
    
    % Initialize the data grid to zeros.
    yearDataGrid = zeros(xLength,yLength);
    winterDataGrid = zeros(xLength,yLength);
    springDataGrid = zeros(xLength,yLength);
    summerDataGrid = zeros(xLength,yLength);
    fallDataGrid = zeros(xLength,yLength);
    
    for i=1:length(data.springdata)
        % Convert the latitude and longitude fields for each data point
        % into indices of the array.
        xInd = (data.latitude(i) - latmin)*4 + 1;
        yInd = (data.longitude(i) - lonmin)*4 + 1;
        
        % Add the data to the array
        yearDataGrid(xInd,yInd) = data.yeardata(i);
        winterDataGrid(xInd,yInd) = data.winterdata(i);
        springDataGrid(xInd,yInd) = data.springdata(i);
        summerDataGrid(xInd,yInd) = data.summerdata(i);
        fallDataGrid(xInd,yInd) = data.falldata(i);
    end
    
    yearDataGrid(yearDataGrid == 0) = NaN;
    winterDataGrid(winterDataGrid == 0) = NaN;
    springDataGrid(springDataGrid == 0) = NaN;
    summerDataGrid(summerDataGrid == 0) = NaN;
    fallDataGrid(fallDataGrid == 0) = NaN;
    
    % Create the Spring axes
    subplot(221);
    axes11 = axesm('MapProjection', 'eqaconicstd',...
        'MapLatLimit',[latmin latmax],'MapLonLimit', [lonmin lonmax],...
        'Frame','on');
    geoshow(axes11, lat, long, 'Color', 'black');
    
    % Create contour plot
    % for pretty contours, use the contourm call with 'LevelStep', 2
    
    % I don't know if this actually scales the data properly for a axesm object,
    % there is something buggy about the implementation of axesm that
    % doesn't allow one to set the Clim property directly.  But, this workaround
    % seems to produce the desired effect, allowing for one shared colorbar for all 4 subplots.
    caxis(axes11,[CminSeasonal CmaxSeasonal]);
    set(axes11,'Position',[0.13 0.51 0.44 0.44]);
    %     contourm(springDataGrid, refVec, levels,'Fill','on','Parent',axes11);
    surfm([latmin latmax], [lonmin lonmax], springDataGrid, 'Parent',axes11);
    % Create title
    title({'Spring'});
    tightmap;
    
    % Create the Summer axes
    subplot(222);
    axes12 = axesm('MapProjection', 'eqaconicstd',...
        'MapLatLimit',[latmin latmax],'MapLonLimit', [lonmin lonmax],...
        'Frame','on');
    geoshow(axes12, lat, long, 'Color', 'black');
    
    % Create contour plot
    caxis(axes12,[CminSeasonal CmaxSeasonal]);
    set(axes12,'Position',[0.47 0.51 0.44 0.44]);
    %     contourm(summerDataGrid, refVec, levels,'Fill','on','Parent',axes12);
    surfm([latmin latmax], [lonmin lonmax], summerDataGrid, 'Parent',axes12);
    
    % Create title
    title({'Summer'});
    tightmap;
    
    % Create the Fall axes
    subplot(223);
    axes13 = axesm('MapProjection', 'eqaconicstd',...
        'MapLatLimit',[latmin latmax],'MapLonLimit', [lonmin lonmax],...
        'Frame','on');
    geoshow(axes13, lat, long, 'Color', 'black');
    
    % Create contour plot
    % for pretty contours, use the contourm call with 'LevelStep', 2
    caxis(axes13,[CminSeasonal CmaxSeasonal]);
    set(axes13,'Position',[0.13 0.01 0.44 0.44]);
    %     contourm(fallDataGrid, refVec, levels,'Fill','on','Parent',axes13);
    surfm([latmin latmax], [lonmin lonmax], fallDataGrid, 'Parent',axes13);
    % Create title
    title({'Fall'});
    tightmap;
    
    % Create the Winter axes
    subplot(224);
    axes14 = axesm('MapProjection', 'eqaconicstd',...
        'MapLatLimit',[latmin latmax],'MapLonLimit', [lonmin lonmax],...
        'Frame','on');
    geoshow(axes14, lat, long, 'Color', 'black');
    
    % Create contour plot
    % for pretty contours, use the contourm call with 'LevelStep', 2
    
    caxis(axes14,[CminSeasonal CmaxSeasonal]);
    set(axes14,'Position',[0.47 0.01 0.44 0.44]);
    %     contourm(winterDataGrid, refVec, levels,'Fill','on','Parent',axes14);
    surfm([latmin latmax], [lonmin lonmax], winterDataGrid, 'Parent',axes14);
    % Create title
    title({'Winter'});
    tightmap;
    
    % Create colorbar
    % A colorbar is created in a custom position to the right of all
    % the subplots.  Since the colormap is associated with the current axes
    % (sub4), that axes must be set back to its original size and position
    % after creating the colormap (i.e. adding colormaps resize axes
    % automatically, so the position is saved beforehand and reset)
    %     sub4Pos=get(sub4, 'Position');
    
    %appears to be a bug in contourcbar, causing the top range of colors to
    %be incorrect if the full range of data is not represented in axes14
    %(which it won't be since it is winter), so use the standard colorbar
    %instead.  This won't then correspond exactly to the levels in contourm
    %plot
    %     cbarSeasonal = colorbar('peer',axes14,'Position', [.88 .1083 .0476 .8143]);
    cbarSeasonal = colorbar('peer',axes14,'Position', [0.87 0.03 0.04 0.9]);
    %     set(sub4,'Position',sub4Pos);
    ylabel(cbarSeasonal, clabel);
    
    % Create the second figure, for the yearly data
    figYearly = figure('NumberTitle','off',...
        'Name',['Yearly ' titletext]);
    axes2 = axesm('MapProjection', 'eqaconicstd',...
        'MapLatLimit',[latmin latmax],'MapLonLimit', [lonmin lonmax],...
        'Frame','on');
    %     set(axes2,'Parent',figYearly);
    geoshow(axes2, lat, long, 'Color', 'black');
    
    % Create contour plot
    % for pretty contours, use the contourm call with 'LevelStep', 2
    caxis(axes2, [CminYearly CmaxYearly]);
    set(axes2,'Position',[0.01 0.01 0.98 0.98]);
    %     contourm(yearDataGrid, refVec, levels,'Fill','on','Parent',axes2);
    surfm([latmin latmax], [lonmin lonmax], yearDataGrid, 'Parent',axes2);
    
    % Create colorbar
    cbar2 = colorbar('peer',axes2);
    set(cbar2,'Position',[0.87 0.03 0.04 0.9]);
    ylabel(cbar2, clabel);
    tightmap
else
    error('Invalid type of plot specified: please choose "US", or "EU"')
end

% Save the figures to .png image files
saveas(figSeasonal,['Seasonal ' titletext '.tif'])
saveas(figSeasonal,['Seasonal ' titletext '.png'])
saveas(figSeasonal,['Seasonal ' titletext '.fig'])
saveas(figYearly,['Yearly ' titletext '.tif'])
saveas(figYearly,['Yearly ' titletext '.png'])
saveas(figYearly,['Yearly ' titletext '.fig'])
end