classdef Simulation < handle
    % SIMULATION creates a simulation for calculating and comparing different
    % DCS-CHP systems.
    
    properties %(Access = Private)
        SystemArray
        InputArray %Cell Array - Th, Ph, Pl - each entry is double
        SystemNames
        EnvArray
        EnvNames
        Figure
        Location
        Results
    end
    
    methods
        
        function sim = Simulation
            sim.SystemArray = {};
            sim.InputArray = {};
            sim.SystemNames = {};
            sim.EnvArray = {};
            sim.EnvNames = {};
            sim.Location = {};
            sim.Results = {};
            sim.gui
        end %end constructor
        
        
        function calculate(sim, sysSelected, envSelected, outputSitesGDX, outputTechGDX)
            if (length(sysSelected) < 1) || (length(envSelected) < 1)
                errordlg('Select at least one system and one environment to calculate')
                return
            end
            % CALCULATE will produce expected output for any number of
            % system-environment combinations by specifying a list of
            % selected systems and a list of selected environments.
            
            CtoK = 273.15;
            kilo = 1e3;
            Megatokilo = 1e3;
            kiloSquared = 1e6;
            yearResults = cell(length(sysSelected),length(envSelected));
            winterResults = cell(length(sysSelected),length(envSelected));
            springResults = cell(length(sysSelected),length(envSelected));
            summerResults = cell(length(sysSelected),length(envSelected));
            fallResults = cell(length(sysSelected),length(envSelected));
            
            if outputSitesGDX
                % ELIN output preprocessing
                numTimeSteps = length(sim.EnvArray{envSelected(1)}.Data.globalhorizontal);
                % preallocate memory for hourly electrical production for ELIN
                elinSolEl = zeros(length(sysSelected), length(envSelected), numTimeSteps);
                elinSolHeat = zeros(length(sysSelected), length(envSelected), numTimeSteps);
                
                %             % create a cell array of row labels hour1, hour2, ...
                %             hoursCell = cellstr(num2str((1:numTimeSteps)','hour%d'));
            end
            
            
            for i = 1:length(sysSelected)
                
                % Check the type of system before performing thermodynamic
                % cycle calculations.
                if isa(sim.SystemArray{sysSelected(i)}, 'CHPSystem')
                    [specificHeatIn, specificHeatOut, etaRankine, ~] = sim.SystemArray{sysSelected(i)}.calculate...
                        (sim.InputArray{sysSelected(i)}(1),...
                        sim.InputArray{sysSelected(i)}(2),...
                        sim.InputArray{sysSelected(i)}(3));
                    
                    % Calculate rated power (Wp) for system and set Rankine
                    % cycle efficiency property for the system.
                    sim.SystemArray{sysSelected(i)}.calculateWp(etaRankine);
                    
                elseif isa(sim.SystemArray{sysSelected(i)}, 'ThermalSystem')
                    [specificHeatIn, specificHeatOut] = sim.SystemArray{sysSelected(i)}.calculate...
                        (sim.InputArray{sysSelected(i)}(1),...
                        sim.InputArray{sysSelected(i)}(2),...
                        sim.InputArray{sysSelected(i)}(3));
                end
                
                for j = 1:length(envSelected)
                    % Ambient temperature and solar insolation varies hour-by-hour, so
                    % efficiency of collector does also.
                    
                    % tracking collector dimension is 1 for 1-axis tracking and 2 for 2-axis
                    if (sim.SystemArray{sysSelected(i)}.TrackingAxes == 2)
                        %                         isa(sim.SystemArray{sysSelected(i)}.Collector, 'ConcentratingThermalCollector') || isa(sim.SystemArray{sysSelected(i)}.Collector, 'PVTCollector') || isa(sim.SystemArray{sysSelected(i)}.Collector, 'CPVCollector') || isa(sim.SystemArray{sysSelected(i)}.Collector, 'CPVCollector')
                        % for 2-axis tracking collectors
                        % tilt is equal to the solar zenith, and the
                        % collector azimuth (panel direction) and solar azimuth are also equal.
                        panelDirection = sim.EnvArray{envSelected(j)}.Sun.azimuth;
                        %                         panelTilt = 90 - sim.EnvArray{envSelected(j)}.Sun.zenith;
                        panelTilt = sim.EnvArray{envSelected(j)}.Sun.zenith;
                        
                        %                         incDirect = zeros(size(panelTilt));
                        %                         incLong = incDirect;
                        %                         incTrans = incDirect;
                        
                    elseif (sim.SystemArray{sysSelected(i)}.TrackingAxes == 1)
                        % for 1-axis tracking collectors, assume tilt equal
                        % to latitude and set collector azimuth (panel direction) equal to solar azimuth.
                        panelDirection = sim.EnvArray{envSelected(j)}.Sun.azimuth;
                        panelTilt = sim.EnvArray{envSelected(j)}.Info.latitude;  %Assume collector tilted to latitude is near-optimum, since it is DNI that is being collected.
                        
                    else
                        % non-tracking collector
                        panelDirection = 0; % Assume collector facing due south
                        
                        % Use "optimal" tilt equation, based on correlation
                        % from Chang, 2009 "The sun's apparent position and
                        % the optimal tilt angle of a solar collector in
                        % the northern hemisphere"
                        if sim.EnvArray{envSelected(j)}.Info.latitude <= 65
                            panelTilt = 0.764 * sim.EnvArray{envSelected(j)}.Info.latitude + 2.14;
                        else
                            panelTilt = 0.224 * sim.EnvArray{envSelected(j)}.Info.latitude + 33.65;
                        end
                    end
                    
                    % calculate incidence angles of sunlight on the
                    % collector
                    [incDirect, incLong, incTrans] = sim.incAngles(sim.EnvArray{envSelected(j)}.Sun.azimuth,...
                        sim.EnvArray{envSelected(j)}.Sun.zenith,...
                        panelDirection, panelTilt);
                    
                    
                    % irradianceCalc differentiates between incident irradiance on
                    % concentrating and non-concentrating collectors
                    % (i.e. concentrating only including direct normal
                    % and circumsolar, and non-concentrating including
                    % diffuse and ground-reflected as well)
                    [incIrradiance, Ib, Id, Ig] = sim.irradianceCalc...
                        (sim.SystemArray{sysSelected(i)},...
                        sim.EnvArray{envSelected(j)}.Data.directnormal,...
                        sim.EnvArray{envSelected(j)}.Data.globalhorizontal,...
                        sim.EnvArray{envSelected(j)}.Data.diffusehorizontal,...
                        sim.EnvArray{envSelected(j)}.Data.ETRnormal,...
                        sim.EnvArray{envSelected(j)}.Data.albedocorrected,...
                        incDirect, incLong, incTrans,...
                        sim.EnvArray{envSelected(j)}.Sun.zenith,...
                        panelTilt);
                    
                    % Now perform the efficiency calcs for the systems
                    switch class(sim.SystemArray{sysSelected(i)})
                        % Thermal System
                        case 'ThermalSystem'
                            etaEnvCollectorThermal = sim.SystemArray{sysSelected(i)}.Collector.getEfficiency...
                                (incIrradiance, sim.EnvArray{envSelected(j)}.Data.temperature + CtoK);
                            heatEfficiency = etaEnvCollectorThermal .* sim.SystemArray{sysSelected(i)}.HeatExchanger.Eta;
                            
                            % Calculate time-series output
                            heatOutput = incIrradiance.*heatEfficiency;
                            elecOutput = zeros(size(heatOutput));  %This is a heat-only system
                            
                            % CHP System
                        case 'CHPSystem'
                            etaEnvCollectorThermal = sim.SystemArray{sysSelected(i)}.Collector.getEfficiency...
                                (incIrradiance, sim.EnvArray{envSelected(j)}.Data.temperature + CtoK);
                            heatEfficiency = specificHeatOut / specificHeatIn .* sim.SystemArray{sysSelected(i)}.Condenser.Eta .* etaEnvCollectorThermal;
                            elecEfficiency = etaEnvCollectorThermal .* etaRankine .* sim.SystemArray{sysSelected(i)}.Generator.Eta;
                            
                            % Calculate time-series output
                            heatOutput = incIrradiance.*heatEfficiency;
                            elecOutput = incIrradiance.*elecEfficiency;
                            
                            % PV System
                        case 'PVSystem'
                            if isa(sim.SystemArray{sysSelected(i)}.Collector, 'CPVCollector')
                                etaEnvCollectorPV = sim.SystemArray{sysSelected(i)}.Collector.getEfficiency...
                                    (incIrradiance, sim.EnvArray{envSelected(j)}.Data.temperature, sim.EnvArray{envSelected(j)}.Data.windSpeed, sim.EnvArray{envSelected(j)}.Data.pressure, sim.EnvArray{envSelected(j)}.Sun.zenith);
                            else
                                etaEnvCollectorPV = sim.SystemArray{sysSelected(i)}.Collector.getEfficiency...
                                    (incIrradiance, sim.EnvArray{envSelected(j)}.Data.temperature);
                            end
                            
                            elecEfficiency = etaEnvCollectorPV .* sim.SystemArray{sysSelected(i)}.Inverter.Eta;
                            
                            % Calculate time-series output
                            elecOutput = incIrradiance.*elecEfficiency;
                            heatOutput = zeros(size(elecOutput)); %This is an electricity-only PV system
                            
                            % PVT System
                        case 'PVTSystem'
                            [etaEnvCollectorThermal, etaEnvCollectorPV] = sim.SystemArray{sysSelected(i)}.Collector.getEfficiency...
                                (incIrradiance, sim.EnvArray{envSelected(j)}.Data.temperature + CtoK);
                            heatEfficiency = specificHeatOut / specificHeatIn .* sim.SystemArray{sysSelected(i)}.Condenser.Eta .* etaEnvCollectorThermal;
                            elecEfficiency = etaEnvCollectorThermal .* etaRankine .* sim.SystemArray{sysSelected(i)}.Generator.Eta + etaEnvCollectorPV .* sim.SystemArray{sysSelected(i)}.Inverter.Eta;
                            
                            % Calculate time-series output
                            heatOutput = incIrradiance.*heatEfficiency;
                            elecOutput = incIrradiance.*elecEfficiency;
                            
                        otherwise
                            error('System type not recognized: must be a ThermalSystem, CHPSystem, PVSystem, or PVTSystem')
                    end
                    
                    if outputSitesGDX
                        % Creation of hourly electrical production for input to
                        % an investment model (ELIN)
                        % Each column is the yearly production in watt-hours per
                        % peak rated watt at STC (Wh/Wp)
                        % Note that this is not correct for PVT or
                        % any system where collector Wp is not well defined.
                        elinSolEl(i,j,:) = elecOutput./sim.SystemArray{sysSelected(i)}.WpEl;
                        if ~isa(sim.SystemArray{sysSelected(i)},'PVSystem')
                            elinSolHeat(i,j,:) = heatOutput./sim.SystemArray{sysSelected(i)}.WpTh;
                        end
                    end
                    
                    % test output based on system watt peak.
%                     elecOutput = elecOutput./sim.SystemArray{sysSelected(i)}.WpEl;
%                     if ~isa(sim.SystemArray{sysSelected(i)},'PVSystem')
%                         heatOutput = heatOutput./sim.SystemArray{sysSelected(i)}.WpTh;
%                     end
                    % end test
                    
                    % creation of seasonal and yearly output for
                    % calculating graphs
                    [yearlyHeatOutput, winterHeatOutput, springHeatOutput, summerHeatOutput, fallHeatOutput] = sim.seasonSums(heatOutput);
                    [yearlyElecOutput, winterElecOutput, springElecOutput, summerElecOutput, fallElecOutput] = sim.seasonSums(elecOutput);
                    [yearlyBeamIrr, winterBeamIrr, springBeamIrr, summerBeamIrr, fallBeamIrr] = sim.seasonSums(Ib);
                    [yearlyDiffuseIrr, winterDiffuseIrr, springDiffuseIrr, summerDiffuseIrr, fallDiffuseIrr] = sim.seasonSums(Id);
                    [yearlyGroundReflectedIrr, winterGroundReflectedIrr, springGroundReflectedIrr, summerGroundReflectedIrr, fallGroundReflectedIrr] = sim.seasonSums(Ig);
                    
                    yearResults{i,j} = [yearlyHeatOutput, yearlyElecOutput, yearlyBeamIrr, yearlyDiffuseIrr, yearlyGroundReflectedIrr];
                    winterResults{i,j} = [winterHeatOutput, winterElecOutput, winterBeamIrr, winterDiffuseIrr, winterGroundReflectedIrr];
                    springResults{i,j} = [springHeatOutput, springElecOutput, springBeamIrr, springDiffuseIrr, springGroundReflectedIrr];
                    summerResults{i,j} = [summerHeatOutput, summerElecOutput, summerBeamIrr, summerDiffuseIrr, summerGroundReflectedIrr];
                    fallResults{i,j} = [fallHeatOutput, fallElecOutput, fallBeamIrr, fallDiffuseIrr, fallGroundReflectedIrr];
                end
                
                %                 % now create the table from the array of hourly production
                %                 % data at each location, the EnvNames as variable names,
                %                 % and the hour names just created.  Save the table as a
                %                 % text file.
                %                 elinSolElTable = array2table(elinSolEl,'RowNames',hoursCell,'VariableNames', envVarNames);
                %                 writetable(elinSolElTable,strcat('solar_input_',sim.SystemNames{i},'.inc'), 'FileType', 'text', 'Delimiter', 'space', 'WriteRowNames', true);
                
            end
            sim.Results = {yearResults, winterResults, springResults, summerResults, fallResults};
            
            if outputSitesGDX || outputTechGDX
                % create structures for the system types (set)
                
                % for now technology type output is fixed as the following:  fixed monoSi,
                % CdTe, CIG, 1-a tracking mono-Si, CdTe, CIG (utility
                % scale only), plus 2-a tracking HCPV and 2-a tracking
                % CSP(utility only) (T 623K, Phigh 4000 kPa, Plow 100 kPa,
                % EtaTurbine 0.9, all other parameters default) 
                
                solar_systems.name = 'solar_systems';
                solar_systems.uels = {'fixed_monoSi','fixed_CdTe','fixed_CIGS','tracking_monoSi','tracking_CdTe','tracking_CIGS','tracking_HCPV','tracking_CSP'};
                solar_systems.type = 'set';
                
                solar_type.name = 'solar_type';
                solar_type.uels = {'fixed_building_monoSi','fixed_utility_monoSi','fixed_building_CdTe','fixed_utility_CdTe','fixed_building_CIGS','fixed_utility_CIGS','tracking_monoSi','tracking_CdTe','tracking_CIGS','tracking_HCPV','tracking_CSP'};
                solar_type.type = 'set';
                
                building_solar_type.name = 'building_solar_type';
                building_solar_type.uels = {'fixed_building_monoSi','fixed_building_CdTe','fixed_building_CIGS'};
                building_solar_type.type = 'set';
                
                solar_type_to_solar_systems.name = 'solar_type_to_solar_systems';
                solar_type_to_solar_systems.uels = {solar_type.uels, solar_systems.uels};
                solar_type_to_solar_systems.val = [(1:length(solar_type.uels))', [1; 1; 2; 2; 3; 3; 4; 5; 6; 7; 8]];
                solar_type_to_solar_systems.type = 'set';
                
                wind_systems.name = 'wind_systems';
                wind_systems.uels = {'onshore_wind','offshore_wind'};
                wind_systems.type = 'set';
                
                storage_systems.name = 'storage_systems';
                storage_systems.uels = {'battery','TES'};
                storage_systems.type = 'set';
                
                % Numbers from "land-use requirements for solar power plants in
                % the US", NREL (ratio of Direct Area per system capacity to Total Area per system
                % capacity (AC)).  Looking at raw data the average ratio is
                % 0.71 for 2-axis tracking and 0.74 for fixed and 1-axis for all plants that have both direct and
                % total area datapoints.
                fixedLandRatio = 0.74;
                flat1aTrackingLandRatio = 0.74;
                %                 flat2aTrackingLandRatio = 0.71;
                concentrating1aTrackingLandRatio = 0.74;
                concentrating2aTrackingLandRatio = 0.71;
                
%                 % Set the Rankine cycle efficiency of the CSP system for
%                 % use in the 
%                 CSP_EtaRankine.name = 'CSP_EtaRankine';
%                 CSP_EtaRankine.val = [sim.SystemArray{sysSelected(8)}.EtaThermaltoEl];
%                 CSP_EtaRankine.uels = {'tracking_CSP'};
%                 CSP_EtaRankine.type = 'parameter';
%                 CSP_EtaRankine.dim = 0;
%                 CSP_EtaRankine.form = 'full';
                
                % Units are system GWp / km^2 land area (= kW/m^2)
                solar_cap_intensity.name = 'solar_cap_intensity';
                solar_cap_intensity.val = [fixedLandRatio.*sim.SystemArray{sysSelected(1)}.WpEl,...
                    fixedLandRatio.*sim.SystemArray{sysSelected(1)}.WpEl,...
                    fixedLandRatio.*sim.SystemArray{sysSelected(2)}.WpEl,...
                    fixedLandRatio.*sim.SystemArray{sysSelected(2)}.WpEl,...
                    fixedLandRatio.*sim.SystemArray{sysSelected(3)}.WpEl,...
                    fixedLandRatio.*sim.SystemArray{sysSelected(3)}.WpEl,...
                    flat1aTrackingLandRatio.*sim.SystemArray{sysSelected(4)}.WpEl,...
                    flat1aTrackingLandRatio.*sim.SystemArray{sysSelected(5)}.WpEl,...
                    flat1aTrackingLandRatio.*sim.SystemArray{sysSelected(6)}.WpEl,...
                    concentrating2aTrackingLandRatio.*sim.SystemArray{sysSelected(7)}.WpEl,...
                    concentrating1aTrackingLandRatio.*sim.SystemArray{sysSelected(8)}.WpEl]./kilo;
                solar_cap_intensity.uels = {solar_type.uels};
                solar_cap_intensity.type = 'parameter';
                solar_cap_intensity.dim = 1;
                solar_cap_intensity.form = 'full';
                
                % NPSol_Ic, EPSol_OM (Investment cost with exponential learning curve model implemented in GAMS, and OM cost of which only fixed OM is
                % applicable to solar)  Costs from 2012, 2020 and 2035 from IEA World Energy Outlook WEIO2014PGAssumptions (2014).
                % Exchange rate from USD to EUR for 2012 from same report.
                
                USDtoEUR = 0.78;
                timeIEA = [2012;2020;2035];
                baseInvestCostBuildingPV = [3250; 2350; 1910]*USDtoEUR;
                baseInvestCostUtilityPV = [2490; 1780; 1440]*USDtoEUR;
                baseOMcostBuildingPV = [33; 30; 29]*USDtoEUR; % corrected this value so there was a decrease between 2020 and 2035
                baseOMcostUtilityPV = [25; 23; 22]*USDtoEUR;
                
                investCostOnshoreWind = [1790; 1700; 1630]*USDtoEUR;
                investCostOffshoreWind = [5180; 4000; 3310]*USDtoEUR;
                OMcostOnshoreWind = [46; 43; 41]*USDtoEUR;
                OMcostOffshoreWind = [181; 140; 116]*USDtoEUR;
                
                %Tracking costs from Solar Balance-of-System: To Track or Not to
                %Track, Part I (2013)
                costBOS = 0.95*0.75*kilo; % in EUR/kW: 0.75 is USD/EUR exchange rate 2013, 0.95 is BOS costs Europe in USD/W
                costBOSTracking = (1.17+0.36)/1.17*costBOS; %Additional cost added to BOS for tracking
                %Increased cost of tracking factor in terms of system cost
                trackingCostFactor = (baseInvestCostUtilityPV-costBOS+costBOSTracking)./baseInvestCostUtilityPV;
                
                investCostFixedBuildingPV = baseInvestCostBuildingPV;
                investCostFixedUtilityPV = baseInvestCostUtilityPV;
                investCostFlatTrackingPV = trackingCostFactor.*investCostFixedUtilityPV;
                investCostHCPV = baseInvestCostUtilityPV*1.5; % Ratio of cost compared to utility PV from Fraunhofer Levelised cost of electricity renewable energies (2014)
                investCostCSP = [7250; 5880; 4580]*USDtoEUR;
                OMcostFixedBuildingPV = baseOMcostBuildingPV;
                OMcostFixedUtilityPV = baseOMcostUtilityPV;
                OMcostFlatTrackingPV = trackingCostFactor.*OMcostFixedUtilityPV;
                OMcostHCPV = baseOMcostUtilityPV*1.5; % Ratio of cost compared to utility PV from Fraunhofer Levelised cost of electricity renewable energies (2014)
                OMcostCSP = [290; 235; 183]*USDtoEUR;
                
                % market leaders trajectory of battery costs from Nykvist, Bjrn, Nilsson Mns "Rapidly
                % falling costs of battery packs for electric vehicles".
                % Nature and Climate change 2015.
                timeNatureBattery = [2010;2014;2030];
                investCostBattery = [400;300;150]*USDtoEUR*Megatokilo;
                
                % costs for Thermal Energy Storage (TES) for molten salt CSP from "Solar Industrial Initiative for STE -
                % Implementation Plan 2013-2015"
                timeSTERoadmap = [2010;2015;2025];
                investCostTEStoEl = [35; 20; 15]/sim.SystemArray{sysSelected(8)}.EtaThermaltoEl*Megatokilo;
                
                % find best fit exponential learning curves based on cost data
                % for both fixed and O&M costs
                fitInvestCostFixedBuildingPV = fit(timeIEA,investCostFixedBuildingPV,'exp1');
                fitInvestCostFixedUtilityPV = fit(timeIEA,investCostFixedUtilityPV,'exp1');
                fitInvestCostFlatTrackingPV = fit(timeIEA,investCostFlatTrackingPV,'exp1');
                fitInvestCostHCPV = fit(timeIEA,investCostHCPV,'exp1');
                fitInvestCostCSP = fit(timeIEA,investCostCSP,'exp1');
                fitOMcostFixedBuildingPV = fit(timeIEA,OMcostFixedBuildingPV,'exp1');
                fitOMcostFixedUtilityPV = fit(timeIEA,OMcostFixedUtilityPV,'exp1');
                fitOMcostFlatTrackingPV = fit(timeIEA,OMcostFlatTrackingPV,'exp1');
                fitOMcostHCPV = fit(timeIEA,OMcostHCPV,'exp1');
                fitOMcostCSP = fit(timeIEA,OMcostCSP,'exp1');
                fitInvestCostOnshoreWind = fit(timeIEA,investCostOnshoreWind,'exp1');
                fitInvestCostOffshoreWind = fit(timeIEA,investCostOffshoreWind,'exp1');
                fitOMcostOnshoreWind = fit(timeIEA,OMcostOnshoreWind,'exp1');
                fitOMcostOffshoreWind = fit(timeIEA,OMcostOffshoreWind,'exp1');
                fitInvestCostBattery = fit(timeNatureBattery,investCostBattery,'exp1');
                fitInvestCostTES = fit(timeSTERoadmap, investCostTEStoEl,'exp1');
                
                solar_invest_cost_coef_a.name = 'solar_invest_cost_coef_a';
                solar_invest_cost_coef_a.val = [fitInvestCostFixedBuildingPV.a,fitInvestCostFixedUtilityPV.a,fitInvestCostFixedBuildingPV.a,fitInvestCostFixedUtilityPV.a,fitInvestCostFixedBuildingPV.a,fitInvestCostFixedUtilityPV.a,fitInvestCostFlatTrackingPV.a,fitInvestCostFlatTrackingPV.a,fitInvestCostFlatTrackingPV.a,fitInvestCostHCPV.a,fitInvestCostCSP.a];
                solar_invest_cost_coef_a.uels = {solar_type.uels};
                solar_invest_cost_coef_a.type = 'parameter';
                solar_invest_cost_coef_a.dim = 1;
                solar_invest_cost_coef_a.form = 'full';
                
                solar_fixed_OM_cost_coef_a.name = 'solar_fixed_OM_cost_coef_a';
                solar_fixed_OM_cost_coef_a.val = [fitOMcostFixedBuildingPV.a,fitOMcostFixedUtilityPV.a,fitOMcostFixedBuildingPV.a,fitOMcostFixedUtilityPV.a,fitOMcostFixedBuildingPV.a,fitOMcostFixedUtilityPV.a,fitOMcostFlatTrackingPV.a,fitOMcostFlatTrackingPV.a,fitOMcostFlatTrackingPV.a,fitOMcostHCPV.a,fitOMcostCSP.a];
                solar_fixed_OM_cost_coef_a.uels = {solar_type.uels};
                solar_fixed_OM_cost_coef_a.type = 'parameter';
                solar_fixed_OM_cost_coef_a.dim = 1;
                solar_fixed_OM_cost_coef_a.form = 'full';
                
                solar_invest_cost_coef_b.name = 'solar_invest_cost_coef_b';
                solar_invest_cost_coef_b.val = [fitInvestCostFixedBuildingPV.b,fitInvestCostFixedUtilityPV.b,fitInvestCostFixedBuildingPV.b,fitInvestCostFixedUtilityPV.b,fitInvestCostFixedBuildingPV.b,fitInvestCostFixedUtilityPV.b,fitInvestCostFlatTrackingPV.b,fitInvestCostFlatTrackingPV.b,fitInvestCostFlatTrackingPV.b,fitInvestCostHCPV.b,fitInvestCostCSP.b];
                solar_invest_cost_coef_b.uels = {solar_type.uels};
                solar_invest_cost_coef_b.type = 'parameter';
                solar_invest_cost_coef_b.dim = 1;
                solar_invest_cost_coef_b.form = 'full';
                
                solar_fixed_OM_cost_coef_b.name = 'solar_fixed_OM_cost_coef_b';
                solar_fixed_OM_cost_coef_b.val = [fitOMcostFixedBuildingPV.b,fitOMcostFixedUtilityPV.b,fitOMcostFixedBuildingPV.b,fitOMcostFixedUtilityPV.b,fitOMcostFixedBuildingPV.b,fitOMcostFixedUtilityPV.b,fitOMcostFlatTrackingPV.b,fitOMcostFlatTrackingPV.b,fitOMcostFlatTrackingPV.b,fitOMcostHCPV.b,fitOMcostCSP.b];
                solar_fixed_OM_cost_coef_b.uels = {solar_type.uels};
                solar_fixed_OM_cost_coef_b.type = 'parameter';
                solar_fixed_OM_cost_coef_b.dim = 1;
                solar_fixed_OM_cost_coef_b.form = 'full';
                
                wind_invest_cost_coef_a.name = 'wind_invest_cost_coef_a';
                wind_invest_cost_coef_a.val = [fitInvestCostOnshoreWind.a,fitInvestCostOffshoreWind.a];
                wind_invest_cost_coef_a.uels = {wind_systems.uels};
                wind_invest_cost_coef_a.type = 'parameter';
                wind_invest_cost_coef_a.dim = 1;
                wind_invest_cost_coef_a.form = 'full';
                
                wind_fixed_OM_cost_coef_a.name = 'wind_fixed_OM_cost_coef_a';
                wind_fixed_OM_cost_coef_a.val = [fitOMcostOnshoreWind.a,fitOMcostOffshoreWind.a];
                wind_fixed_OM_cost_coef_a.uels = {wind_systems.uels};
                wind_fixed_OM_cost_coef_a.type = 'parameter';
                wind_fixed_OM_cost_coef_a.dim = 1;
                wind_fixed_OM_cost_coef_a.form = 'full';
                
                wind_invest_cost_coef_b.name = 'wind_invest_cost_coef_b';
                wind_invest_cost_coef_b.val = [fitInvestCostOnshoreWind.b,fitInvestCostOffshoreWind.b];
                wind_invest_cost_coef_b.uels = {wind_systems.uels};
                wind_invest_cost_coef_b.type = 'parameter';
                wind_invest_cost_coef_b.dim = 1;
                wind_invest_cost_coef_b.form = 'full';
                
                wind_fixed_OM_cost_coef_b.name = 'wind_fixed_OM_cost_coef_b';
                wind_fixed_OM_cost_coef_b.val = [fitOMcostOnshoreWind.b,fitOMcostOffshoreWind.b];
                wind_fixed_OM_cost_coef_b.uels = {wind_systems.uels};
                wind_fixed_OM_cost_coef_b.type = 'parameter';
                wind_fixed_OM_cost_coef_b.dim = 1;
                wind_fixed_OM_cost_coef_b.form = 'full';
                
                storage_invest_cost_coef_a.name = 'storage_invest_cost_coef_a';
                storage_invest_cost_coef_a.val = [fitInvestCostBattery.a,fitInvestCostTES.a];
                storage_invest_cost_coef_a.uels = {storage_systems.uels};
                storage_invest_cost_coef_a.type = 'parameter';
                storage_invest_cost_coef_a.dim = 1;
                storage_invest_cost_coef_a.form = 'full';
                
                
                storage_invest_cost_coef_b.name = 'storage_invest_cost_coef_b';
                storage_invest_cost_coef_b.val = [fitInvestCostBattery.b,fitInvestCostTES.b];
                storage_invest_cost_coef_b.uels = {storage_systems.uels};
                storage_invest_cost_coef_b.type = 'parameter';
                storage_invest_cost_coef_b.dim = 1;
                storage_invest_cost_coef_b.form = 'full';
                
                wgdx('solar_info.gdx', solar_systems, solar_type, building_solar_type, solar_type_to_solar_systems, solar_cap_intensity, solar_invest_cost_coef_a, solar_invest_cost_coef_b, solar_fixed_OM_cost_coef_a, solar_fixed_OM_cost_coef_b);
                
                wgdx('wind_info.gdx', wind_systems, wind_invest_cost_coef_a, wind_invest_cost_coef_b, wind_fixed_OM_cost_coef_a, wind_fixed_OM_cost_coef_b);
                
                wgdx('storage_info.gdx', storage_systems, storage_invest_cost_coef_a, storage_invest_cost_coef_b);
            end
            
            if outputSitesGDX
                % gridSpacing in degrees latitude and longitude, minimum
                % 0.25 in multiples of 0.25.
                gridSpacing = 0.5;
                
                quadArea = zeros(length(envSelected),1);
                quadAreaNatura2000 = zeros(length(envSelected),1);
                quadAreaWater = zeros(length(envSelected),1);
                quadAreaUrban = zeros(length(envSelected),1);
                %                 latGeoQuads = zeros(length(envSelected),1);
                %                 lonGeoQuads = zeros(length(envSelected),1);
                %                 earthEllipsoid = referenceSphere('earth','km');
                whichNuts2Index = zeros(length(envSelected),1);
                gridSpacingIndex = zeros(length(envSelected),1);
                envVarNames = cell(1,length(envSelected));
                % Newer 2010 data not compatible with existing ELIN
                %nuts2Regions = shaperead('NUTS_RG_03M_2010.shp', 'Attributes', {'NUTS_ID'}, 'Selector', {@(v1) v1==2, 'STAT_LEVL_'}, 'UseGeoCoords', true);
                nuts2Regions = shaperead('NUTS_RG_10M_2006.shp', 'Attributes', {'NUTS_ID'}, 'Selector', {@(v1) v1==2, 'STAT_LEVL_'}, 'UseGeoCoords', true);
                % Newer version from European Environment Agency
                %                 natura2000Regions = shaperead('Natura2000_end2013_rev1.shp');
                natura2000Regions = shaperead('Natura2000_End2011_EU_Clip_Project1.shp');
                
                urbanRegions = shaperead('urbanEsri_Europe_Clip.shp','UseGeoCoords', true);
                waterRegions = shaperead('waterEsri_Europe_Clip.shp','UseGeoCoords', true);
                
                % the default projection for natura 2000 data is the
                % lambert azimuthal equal area projection (EPSG:3035).  We need to
                % convert this to standard lat/lon geographic
                % system.
                % first define the lambert azimuthal projection and
                % setup default properties.
                mstruct = defaultm('eqaazim');
                mstruct = defaultm(mstruct);
                
                % then update the map projection structure to
                % set properties from the given  projection
                
                % set the earth ellipsoid to use with units in meters
                earthEllipsoid = referenceEllipsoid('grs80', 'm');
                
                mstruct.origin = [52 10 0];
                mstruct.geoid = earthEllipsoid;
                mstruct.falseeasting = 4321000;
                mstruct.falsenorthing = 3210000;
                
                %parfor goes here for faster processing
                parfor j = 1:length(envSelected)
                    
                    % only use one quarter of the total environments, or an array
                    % spacing of half degree longitude and latitude instead of
                    % quarter degree.
                    fractionLat = mod(sim.EnvArray{envSelected(j)}.Info.latitude,1);
                    fractionLon = mod(sim.EnvArray{envSelected(j)}.Info.longitude,1);
                    
                    if ((fractionLat==0.375)||(fractionLat==0.375+gridSpacing))&&((fractionLon==0.375)||(fractionLon==0.375+gridSpacing))
                        gridSpacingIndex(j) = 1;
                        
                        % classify each environment location in its NUTS2 region by
                        % testing whether the longitude and latitude are inside the
                        % borders of the region.
                        for k=1:length(nuts2Regions)
                            if inpolygon(sim.EnvArray{envSelected(j)}.Info.latitude, sim.EnvArray{envSelected(j)}.Info.longitude, nuts2Regions(k).Lat, nuts2Regions(k).Lon)
                                whichNuts2Index(j) = k;
                                %                         nuts2Regions(k).NUTS_ID;
                                break;
                            end
                        end
                        
                        % only further process this environment location if it is inside a  NUTS2 Region, ELIN doesn't care about anything outside NUTS2 regions
                        if whichNuts2Index(j)
                            % make the grid quadrangle, assuming the environments
                            % lat-lon point is in the center of the quadrangle
                            [latGeoQuad, lonGeoQuad] = outlinegeoquad([sim.EnvArray{envSelected(j)}.Info.latitude - gridSpacing/2, sim.EnvArray{envSelected(j)}.Info.latitude + gridSpacing/2], [sim.EnvArray{envSelected(j)}.Info.longitude - gridSpacing/2, sim.EnvArray{envSelected(j)}.Info.longitude + gridSpacing/2],0.5,0.5);
                            quadArea(j) = areaint(latGeoQuad, lonGeoQuad, earthEllipsoid);
                            %                     quadArea(j) = areaquad(sim.EnvArray{envSelected(j)}.Info.latitude, sim.EnvArray{envSelected(j)}.Info.longitude, sim.EnvArray{envSelected(j)}.Info.latitude + gridSpacing, sim.EnvArray{envSelected(j)}.Info.longitude + gridSpacing, earthEllipsoid);
                            
                            % Make a valid environment name and truncate to 16 characters for GAMS
                            % compatibility
                            % Make variable names for the hourly input that is just
                            % a concatenated latitude and longitude cell array.
                            %                 envVarNames{1,j} = matlab.lang.makeValidName(num2str(j));
                            envVarNames{1,j} = matlab.lang.makeValidName(strcat('N',num2str(sim.EnvArray{envSelected(j)}.Info.latitude),'E',num2str(sim.EnvArray{envSelected(j)}.Info.longitude)));
                            %                 if length(sim.EnvNames{envSelected(j)})>15
                            %                     envVarNames{j} = matlab.lang.makeValidName(sim.EnvNames{envSelected(j)}(1:15));
                            %                 else
                            %                     envVarNames{j} = matlab.lang.makeValidName(sim.EnvNames{envSelected(j)});
                            %                 end
                            
                            % calculate the area for each natura2000 region, water, and urban regions in each quadrangle
                            for k=1:length(natura2000Regions)
                                % check intersection of every natura2000 region with
                                % the quadrangle formed by grid spacing
                                [latNatura2000, lonNatura2000] = minvtran(mstruct, natura2000Regions(k).X, natura2000Regions(k).Y);
                                [intersectLon, intersectLat] = polybool('intersection', lonNatura2000, latNatura2000, lonGeoQuad, latGeoQuad);
                                % sum the area of the intersection for all natura2000 regions
                                % at least partially within the quadrangle.
                                % (if there is an intersection).
                                if ~isempty(intersectLat)
                                    quadAreaNatura2000(j) = quadAreaNatura2000(j) + sum(areaint(intersectLat, intersectLon, earthEllipsoid));
                                end
                            end
                            
                            for k=1:length(urbanRegions)
                                [intersectLon, intersectLat] = polybool('intersection', urbanRegions(k).Lon, urbanRegions(k).Lat, lonGeoQuad, latGeoQuad);
                                % sum the area of the intersection for all urban regions
                                % at least partially within the quadrangle.
                                % (if there is an intersection).
                                if ~isempty(intersectLat)
                                    quadAreaUrban(j) = quadAreaUrban(j) + sum(areaint(intersectLat, intersectLon, earthEllipsoid));
                                end
                            end
                            
                            for k=1:length(waterRegions)
                                [intersectLon, intersectLat] = polybool('intersection', waterRegions(k).Lon, waterRegions(k).Lat, lonGeoQuad, latGeoQuad);
                                % sum the area of the intersection for all water regions
                                % at least partially within the quadrangle.
                                % (if there is an intersection).
                                if ~isempty(intersectLat)
                                    quadAreaWater(j) = quadAreaWater(j) + sum(areaint(intersectLat, intersectLon, earthEllipsoid));
                                end
                            end
                        end
                    end
                    
                end
                
                % all zeros in whichNuts2Regions are environments that aren't
                % located in any nuts2 region, so we don't want to write these
                % to the gdx file for ELIN.  Also only use the latitudes and
                % longitudes that belong to to the grid spacing selected.
                
                inNuts2 = (~(whichNuts2Index==0))&(gridSpacingIndex==1);
                %             whichNuts2Region(cellfun(@isempty, whichNuts2Region))={'none'};
                
                %             % convert the area array to a table and then write to file.
                %             elinAreaTable = array2table(quadArea, 'RowNames', envVarNames);
                %             writetable(elinAreaTable, 'solar_pot.inc', 'FileType', 'text', 'Delimiter', 'space', 'WriteRowNames', true, 'WriteVariableNames', false);
                
                % Create structures for the location names (set), which regions
                % the sites belong too (set of tuples) and their potentials (parameter).  Write these
                % structures to a *.gdx file for input to GAMS.  Syntax
                % from gdxmrw.pdf and from gams website.
                
                solar_sites.name = 'solar_sites';
                solar_sites.uels = envVarNames(inNuts2);
                solar_sites.type = 'set';
                
                solar_sites_to_nuts2.name = 'solar_sites_to_nuts2';
                solar_sites_to_nuts2.uels = {solar_sites.uels, {nuts2Regions.NUTS_ID}};
                solar_sites_to_nuts2.val = [(1:length(envVarNames(inNuts2)))', whichNuts2Index(inNuts2)];
                solar_sites_to_nuts2.type = 'set';
                
                % We assume building solar can be installed on all land
                % in urban areas.
                solar_pot_buildings.name = 'solar_pot_buildings';
                solar_pot_buildings.val = quadAreaUrban(inNuts2)'/kiloSquared;
                solar_pot_buildings.uels = {solar_sites.uels};
                solar_pot_buildings.type = 'parameter';
                solar_pot_buildings.dim = 1;
                solar_pot_buildings.form = 'full';
                
                % We assume centralized solar can be installed everywhere except
                % natura2000 areas, urban areas, and water areas.
                solar_pot_centralized.name = 'solar_pot_centralized';
                solar_pot_centralized.val = (quadArea(inNuts2)-quadAreaUrban(inNuts2)-quadAreaWater(inNuts2)-quadAreaNatura2000(inNuts2))'/kiloSquared;
                solar_pot_centralized.uels = {solar_sites.uels};
                solar_pot_centralized.type = 'parameter';
                solar_pot_centralized.dim = 1;
                solar_pot_centralized.form = 'full';
                
                % create a cell array of timesteps e.g. hour1 hour2 ...
                hoursCell = cellfun(@(n)(num2str(n,'hour%d')), num2cell((1:numTimeSteps)), 'UniformOutput', false);
                
                solar_el_profiles.name = 'solar_el_profiles';
                solar_el_profiles.val = elinSolEl(:,inNuts2,:);
                solar_el_profiles.uels = {solar_systems.uels, solar_sites.uels, hoursCell};
                solar_el_profiles.type = 'parameter';
                solar_el_profiles.form = 'full';
                
                solar_heat_profiles.name = 'solar_heat_profiles';
                solar_heat_profiles.val = elinSolHeat(:,inNuts2,:);
                solar_heat_profiles.uels = {solar_systems.uels, solar_sites.uels, hoursCell};
                solar_heat_profiles.type = 'parameter';
                solar_heat_profiles.form = 'full';
                                
                %             % convert the whichNuts2Region array to a table and then write
                %             % to file.
                %             elinRegionsTable = cell2table(whichNuts2Region, 'RowNames', envVarNames);
                %             writetable(elinRegionsTable, 'solar_input_regions.inc', 'FileType', 'text', 'Delimiter', 'space', 'WriteRowNames', true, 'WriteVariableNames', false);
                %              solElProdName = matlab.lang.makeValidName(strcat('solar_input_',sim.SystemNames{i}));
                
                wgdx('solar_profiles.gdx', solar_sites, solar_sites_to_nuts2, solar_pot_buildings, solar_pot_centralized, solar_el_profiles, solar_heat_profiles);
                %             assignin('base','Results',simResults);
            end
        end %end function calculate
        
        
        function plot(sim, sysSelected, envSelected)
            %PLOT will plot results of selected systems, after each
            %has CALCULATE(d).  sysSelected will be a double vector with the
            %index of each system (in Simulation.SystemArray) to plot.
            %sysSelected = [1 2 3 6], for example, will plot the 1st,
            %2nd, 3rd, and 6th system in SystemArray.
            if length(sysSelected) < 1
                errordlg('At least one system must be selected for a plot')
                return
            end
            [selection,ok] = listdlg('ListString',{'yearly heat output' 'yearly electrical output' 'beam component of irradiance' 'diffuse component of irradiance' 'ground reflected component of irradiance'},...
                'SelectionMode','single',...
                'ListSize',[250 170]);
            if ok == 0
                return
            end
            switch selection
                case 1
                    titleString = 'heat output';
                case 2
                    titleString = 'electrical output';
                case 3
                    titleString = 'beam irradiance';
                case 4
                    titleString = 'diffuse irradiance';
                case 5
                    titleString = 'ground reflected irradiance';
                otherwise
                    errordlg('This output selection is not specified in the list')
            end
            
            for i = 1:length(sysSelected)
                for j = 1:length(sim.Results{1})
                    
                    % Now extract data and save as a structure.
                    data.yeardata(j) = sim.Results{1}{i,j}(selection);
                    data.winterdata(j) = sim.Results{2}{i,j}(selection);
                    data.springdata(j) = sim.Results{3}{i,j}(selection);
                    data.summerdata(j) = sim.Results{4}{i,j}(selection);
                    data.falldata(j) = sim.Results{5}{i,j}(selection);
                    data.longitude(j) = sim.EnvArray{envSelected(j)}.Info.longitude;
                    data.latitude(j) = sim.EnvArray{envSelected(j)}.Info.latitude;
                end
                
                sysString = sim.SystemNames{sysSelected(i)};
                sim.PlotLevels([sysString ' ' titleString ' by location'],sim.Location,-1,-1,data,'Wh/m^2')
            end
            
            %             for i = 1:length(sim.Results{1})
            %                 data.yeardata(i) = sim.Results{1}{i}(selection);
            %                 data.winterdata(i) = sim.Results{2}{i}(selection);
            %                 data.springdata(i) = sim.Results{3}{i}(selection);
            %                 data.summerdata(i) = sim.Results{4}{i}(selection);
            %                 data.falldata(i) = sim.Results{5}{i}(selection);
            %                 data.longitude(i) = sim.EnvArray{envSelected(i)}.Info.longitude;
            %                 data.latitude(i) = sim.EnvArray{envSelected(i)}.Info.latitude;
            %             end
            %             sim.PlotLevels([titleString ' by location'],sim.Location,-1,-1,data, 'Wh/m^2')
        end
        
        function compare(sim, sysSelected, envSelected)
            %COMPARE will compare results of selected systems, after each
            %has CALCULATE(d).  sysSelected will be a double vector with the
            %index of each system (in Simulation.SystemArray) to compare.
            %sysSelected = [1 2 3 6], for example, will compare the 1st,
            %2nd, 3rd, and 6th system in SystemArray, by simply looking at
            %difference in the Results vectors (pairwise, n choose k (2)).
            %So the above example would result in 6 comparisons plotted.
            
            if length(sysSelected) < 2
                errordlg('At least two systems must be selected for comparison')
                return
            end
            [selection,ok] = listdlg('ListString',{'yearly heat output' 'yearly electrical output' 'beam component of irradiance' 'diffuse component of irradiance' 'ground reflected component of irradiance'},...
                'SelectionMode','single',...
                'ListSize',[250 170]);
            if ok == 0
                return
            end
            switch selection
                case 1
                    titleString = 'heat output';
                case 2
                    titleString = 'electrical output';
                case 3
                    titleString = 'beam irradiance';
                case 4
                    titleString = 'diffuse irradiance';
                case 5
                    titleString = 'ground reflected irradiance';
                otherwise
                    errordlg('This output selection is not specified in the list')
            end
            
            % Create a sequentially numbered version sysSelected for the
            % purpose of indexing the sim.Results that were calculated.
            sysSelectedIndexed = 1:length(sysSelected);
            
            % Creates an array of all the possible pairwise combinations
            % (pairwise comparisons) of the selected systems.
            combinations = nchoosek(sysSelectedIndexed,2);
            for i = 1:size(combinations,1)
                
                % changes the index of what system to compare to what
                % system when calculating percent change depending on
                % which is a bigger value.  i.e. (sys1 - sys2) / sys2
                % or (sys2 - sys1) / sys1.  Takes the first environment as
                % reference.
                
                if sim.Results{1}{combinations(i,1),1}(selection) >= sim.Results{1}{combinations(i,2),1}(selection)
                    index1 = 1;
                    index2 = 2;
                else
                    index1 = 2;
                    index2 = 1;
                end
                
                
                for j = 1:length(sim.Results{1})
                    
                    if (sim.Results{1}{combinations(i,index1),j}(selection) == 0) || (sim.Results{1}{combinations(i,index2),j}(selection) == 0) ||...
                            (sim.Results{2}{combinations(i,index1),j}(selection) == 0) || (sim.Results{2}{combinations(i,index2),j}(selection) == 0) ||...
                            (sim.Results{3}{combinations(i,index1),j}(selection) == 0) || (sim.Results{3}{combinations(i,index2),j}(selection) == 0) ||...
                            (sim.Results{4}{combinations(i,index1),j}(selection) == 0) || (sim.Results{4}{combinations(i,index2),j}(selection) == 0) ||...
                            (sim.Results{5}{combinations(i,index1),j}(selection) == 0) || (sim.Results{5}{combinations(i,index2),j}(selection) == 0)
                        warning('Some zero values in Results, comparison will be invalid at these points');
                    end
                    
                    % Now calculate the percent difference of the data to
                    % be compared, and save as a structure.
                    data.yeardata(j) = 100 .* (sim.Results{1}{combinations(i,index1),j}(selection) - sim.Results{1}{combinations(i,index2),j}(selection)) ./ sim.Results{1}{combinations(i,index2),j}(selection);
                    data.winterdata(j) = 100 .* (sim.Results{2}{combinations(i,index1),j}(selection) - sim.Results{2}{combinations(i,index2),j}(selection)) ./ sim.Results{2}{combinations(i,index2),j}(selection);
                    data.springdata(j) = 100 .* (sim.Results{3}{combinations(i,index1),j}(selection) - sim.Results{3}{combinations(i,index2),j}(selection)) ./ sim.Results{3}{combinations(i,index2),j}(selection);
                    data.summerdata(j) = 100 .* (sim.Results{4}{combinations(i,index1),j}(selection) - sim.Results{4}{combinations(i,index2),j}(selection)) ./ sim.Results{4}{combinations(i,index2),j}(selection);
                    data.falldata(j) = 100 .* (sim.Results{5}{combinations(i,index1),j}(selection) - sim.Results{5}{combinations(i,index2),j}(selection)) ./ sim.Results{5}{combinations(i,index2),j}(selection);
                    data.longitude(j) = sim.EnvArray{envSelected(j)}.Info.longitude;
                    data.latitude(j) = sim.EnvArray{envSelected(j)}.Info.latitude;
                    
                    %                     if isnan(data.yeardata(j)) || ~isfinite(data.yeardata(j)) || isnan(data.winterdata(j)) || ~isfinite(data.winterdata(j)) || isnan(data.springdata(j)) || ~isfinite(data.springdata(j)) || isnan(data.summerdata(j)) || ~isfinite(data.summerdata(j)) || isnan(data.falldata(j)) || ~isfinite(data.falldata(j))
                    %                         warning('Invalid result of comparison: result is infinite or NaN')
                    %                     end
                    
                    
                    
                end
                
                % Correct any non numeric data points from any divide by
                % zeros
                
                data.yeardata(isnan(data.yeardata) == 1) = 0;
                data.winterdata(isnan(data.winterdata) == 1) = 0;
                data.springdata(isnan(data.springdata) == 1) = 0;
                data.summerdata(isnan(data.summerdata) == 1) = 0;
                data.falldata(isnan(data.falldata) == 1) = 0;
                
                data.yeardata(~isfinite(data.yeardata) == 1) = 0;
                data.winterdata(~isfinite(data.winterdata) == 1) = 0;
                data.springdata(~isfinite(data.springdata) == 1) = 0;
                data.summerdata(~isfinite(data.summerdata) == 1) = 0;
                data.falldata(~isfinite(data.falldata) == 1) = 0;
                
                sysString = [sim.SystemNames{sysSelected(combinations(i,index1))} ' compared to ' sim.SystemNames{sysSelected(combinations(i,index2))}];
                sim.PlotLevels([sysString ' ' titleString ' by location'],sim.Location,-1,-1,data,'percent (%)')
            end
        end
        
        %         function optimize(simulation)
        %             %I haven't thought too much about this one yet.
        %         end
    end
    methods (Access = private)
        PlotLevels(sim, titletext, type, lolim, hilim, data, clabel);
        [incDirect, incLong, incTrans] = incAngles(sim, sunAzimuth, sunZenith, panelDirection, panelTilt);
        [incIrradiance, Ib, Id, Ig] = irradianceCalc(sim, sys, GdirectNormal, GdiffuseHorizontal, GetrHorizontal, GglobalHorizontal, albedo, incDirect, incLong, incTrans, sunZenith, panelTilt);
        [yearly, winter, spring, summer, fall] = seasonSums(~, hourlyData);
        getSites(sim);
        [status, info, headers, data] = importtmydata(sim, TMYfilename);
        [month, daynum, hour] = processtime(sim, data);
        decl = declination(sim, daynum);
        hrangle = hourangle(sim, daynum, hour, longitude, meridian);
        [azmth, znth] = sunangles(sim, latitude, declination, hourangle);
    end
end
