function getSites(sim)
% modified from Pepe's by Vince.

% This script creates a structure of TMY sites from NREL TMY data.  Uses
% 'importtmydata' function and the TMY comma separated value files, which
% should be in a folder should be entitled 'TMY_Data'.  Each site's
% standard meridian is calculated, and the 'processtime' function is used
% to generate month, day number, and hour values for each data point.
% Finally, the 'sunangles' function is used to add the sun azimuth and
% zenith to each TMY data point.
%
% UNITS
% Units are taken directly from the TMY data--generally degrees, meters,
% watts per meter squared, Celsius for temperature, Pa for pressure.
% Sun angles are in degrees.
%
% VECTORIZATION
% Vectorization used internally for calculations.

% Reset the EnvNames and EnvArray before beginning to add new Environments
sim.EnvNames = {};
sim.EnvArray = {};

%-------------------------------------------------------------------
% IMPORT SITE DATA

switch sim.Location
    case 'USA'
        % There are 1020 sites, each of which is given a site number.
        numSites = 1020;
        % Each site also has an id number, which is used in its filename, ranging
        % from 690150 to 912850.  (Ending at site id 701200 give a test batch of
        % the first ten sites.)
        startid = 690150;
        %         endid = 912850;
        siteid = startid;
        
    case 'EU'
        % There are 12846 sites, each of which is given a site number.
        numSites = 12846;
        % Import the file names of the TMY data from a tab seperated header file called
        % batch_europe_tmy3.txt
        headerFile = importdata('batch_europe_tmy3.txt', '\t', 3);
end

% Loop through every site
h = waitbar(0,'Importing TMY data...');

for i = 1:numSites
    % Create a new environment object in the environment array for each site.
    % This is where the data will be saved to.
    sim.EnvArray{i} = Environment;
    
    switch sim.Location
        case 'USA'
            % Convert the site id into its associated site path and filename (e.g.
            % the first site's filename is 'TMY_Data\690150TY.csv'.)  Open the file
            % with that filename.  If such a file exists, 'fid' is returned as a
            % reference to the file, to be used in closing it later.
            filename = [num2str(siteid) 'TY.csv'];
            fid = fopen(filename);
            
            % If the file exists (if fid is anything but -1) than import its data.
            % Otherwise, no such file exists (since the site id numbers care not
            % contiguous) so skip importing and loop to the next site id
            % until the next valid filename is found.
            while fid == -1
                siteid = siteid + 1;
                filename = [num2str(siteid) 'TY.csv'];
                fid = fopen(filename);
            end
            fclose(fid);
            siteid = siteid + 1;
            
        case 'EU'
            filename = [headerFile.textdata{i+3,1} '.txt'];
    end
    
    [~, info, ~, data] = importtmydata(sim,filename);
    % Fill the structure for the current site with all the relevant
    % data from the TMY file. All column numbers correspond to those in
    % the CSV file--and, in the case of the time series data, the
    % numbers given in the TMY users manual.
    
    % This includes the site info...
    sim.EnvArray{i}.Info.number = i;
    sim.EnvArray{i}.Info.filename = filename;
    sim.EnvArray{i}.Info.idnum = info{1,1};
    sim.EnvArray{i}.Info.name = info{1,2}{1};
    sim.EnvArray{i}.Info.state = info{1,3}{1};
    sim.EnvArray{i}.Info.timezone = info{1,4};
    sim.EnvArray{i}.Info.meridian = sim.EnvArray{i}.Info.timezone * 15;
    sim.EnvArray{i}.Info.latitude = info{1,5};
    sim.EnvArray{i}.Info.longitude = info{1,6};
    sim.EnvArray{i}.Info.elevation = info{1,7};
    
    % Also add site names to the EnvNames property
    sim.EnvNames{i} = sim.EnvArray{i}.Info.name;
    
    % ...and its time stamps, using the 'processtime' function...
    [sim.EnvArray{i}.Time.month,...
        sim.EnvArray{i}.Time.daynum,...
        sim.EnvArray{i}.Time.hour] ...
        = processtime(sim,data);
    
    % ...and the relevant time series data.
    sim.EnvArray{i}.Data.ETRhorizontal = single(data{1,3});
    sim.EnvArray{i}.Data.ETRnormal = single(data{1,4});
    sim.EnvArray{i}.Data.globalhorizontal = single(data{1,5});
    sim.EnvArray{i}.Data.directnormal = single(data{1,8});
    sim.EnvArray{i}.Data.diffusehorizontal = single(data{1,11});
    sim.EnvArray{i}.Data.temperature = single(data{1,32});
    sim.EnvArray{i}.Data.pressure = single(data{1,41});
    sim.EnvArray{i}.Data.windSpeed = single(data{1,47});
    sim.EnvArray{i}.Data.albedo = single(data{1,62});
    
    %-------------------------------------------------------------------
    % ADD SUN POSITION DATA
    
    % Now that all the TMY data has been imported, it is ammeded with the sun
    % position at every data point.
    
    % For each site, add vectors of the sun azimuth and zenith, using the
    % 'sunangles' function.
    [sim.EnvArray{i}.Sun.azimuth, sim.EnvArray{i}.Sun.zenith] = sunangles(...
        sim, sim.EnvArray{i}.Info.latitude,...
        declination(sim, sim.EnvArray{i}.Time.daynum),...
        hourangle(sim, sim.EnvArray{i}.Time.daynum,...
        sim.EnvArray{i}.Time.hour - 0.5,...
        sim.EnvArray{i}.Info.longitude,...
        sim.EnvArray{i}.Info.meridian));
    
    %-------------------------------------------------------------------
    % CORRECT ALBEDO VALUES
    
    % Finally, correct instances in which albedo is not given (and is equal to
    % the error value -9900).
    
    % Create a new field for corrected albedo values.
    sim.EnvArray{i}.Data.albedocorrected = sim.EnvArray{i}.Data.albedo;
    
    % For most sites, this is correct as-is.  If the lowest albedo value is
    % less than zero, however, the data needs correction.
    if min(sim.EnvArray{i}.Data.albedo) < 0
        
        % If the largest albedo value is also less than zero, then there
        % are no valid albedo measurements in the data set.  In this case,
        % set the albedo to 0.4, which is about the general average.  (It
        % makes little difference.)
        if max(sim.EnvArray{i}.Data.albedo) < 0
            avg = 0.4;
            
            % Otherwise, there are at least some valid albedo measurements.  In
            % this case, copy the albedo values to a temporary variable, remove
            % error values, and take the average of the remaining values.
        else
            temp = sim.EnvArray{i}.Data.albedo;
            temp(temp < 0) = [];
            avg = mean(temp);
        end
        
        % Now replace all error values with this (calculated or set at 0.4)
        % average.
        sim.EnvArray{i}.Data.albedocorrected(sim.EnvArray{i}.Data.albedo < 0) = avg;
    end
    
    % Update the percent of importing progress waitbar
    waitbar(i/numSites,h)
    %     clc
    %     progress = [num2str(i/numSites*100, '%3.1f') ' %'];
    %     disp(progress)
    
end
close(h);
end

% % Clear all the variables used so far, except the 'site' structure.
% clear sitenum startid endid sites siteid progress filename fid ...
%     status info headers data
%
% %-------------------------------------------------------------------
% % ADD SUN POSITION DATA
%
% % Now that all the TMY data has been imported, it is ammeded with the sun
% % position at every data point.
%
% % Loop through every site.
% for i = 1:length(sim.EnvArray)
%
%     % For each site, add vectors of the sun azimuth and zenith, using the
%     % 'sunangles' function.
%     [sim.EnvArray{i}.Sun.azimuth, sim.EnvArray{i}.Sun.zenith] ...
%         = sunangles( ...
%         sim.EnvArray{i}.Info.latitude, ...
%         declination(sim.EnvArray{i}.Time.daynum), ...
%         hourangle( ...
%         sim.EnvArray{i}.Time.daynum, ...
%         sim.EnvArray{i}.Time.hour - 0.5, ...
%         sim.EnvArray{i}.Info.longitude, ...
%         sim.EnvArray{i}.Info.meridian ...
%         ));
% end
%
% %-------------------------------------------------------------------
% % CORRECT ALBEDO VALUES
%
% % Finally, correct instances in which albedo is not given (and is equal to
% % the error value -9900).
%
% % Loop through every site.
% for i = 1:length(sim.EnvArray)
%
%     % Create a new field for corrected albedo values.
%     sim.EnvArray{i}.Data.albedocorrected = sim.EnvArray{i}.Data.albedo;
%
%     % For most sites, this is correct as-is.  If the lowest albedo value is
%     % less than zero, however, the data needs correction.
%     if min(sim.EnvArray{i}.Data.albedo) < 0
%
%         % If the largest albedo value is also less than zero, then there
%         % are no valid albedo measurements in the data set.  In this case,
%         % set the albedo to 0.4, which is about the general average.  (It
%         % makes little difference.)
%         if max(sim.EnvArray{i}.Data.albedo) < 0
%             avg = 0.4;
%
%             % Otherwise, there are at least some valid albedo measurements.  In
%             % this case, copy the albedo values to a temporary variable, remove
%             % error values, and take the average of the remaining values.
%         else
%             temp = sim.EnvArray{i}.Data.albedo;
%             temp(temp < 0) = [];
%             avg = mean(temp);
%         end
%
%         % Now replace all error values with this (calculated or set at 0.4)
%         % average.
%         sim.EnvArray{i}.Data.albedocorrected(sim.EnvArray{i}.Data.albedo < 0) = avg;
%     end
% end
% envArray = sim.EnvArray;
% end
