function hrangle = hourangle(~, daynum, hour, longitude, meridian)

% Calculates the hour angle given day of the year, local standard time in
% hours, local longitude, and longitude of standard meridian (longitude at
% which local standard time is based).  Hour angle is negative before noon,
% zero at noon, and positive after noon. 
%
% UNITS
% Input and output in degrees.
%
% VECTORIZATION
% Configured for vectorized operations: all inputs (except longitude and
% meridian) and outputs can be vectors of multiple values.
%
% REFERENCES
% Duffie, 1991: "Solar Engineering of Thermal Processes" - p 11-13

% Calculate B-factor to simplify time equation (Duffie p 11-12):
B = (daynum - 1)*2*pi/365;

% Time equation (in minutes) corrects time as it shifts through the year:
timeeqn = 229.2*(0.000075 + 0.001868*cos(B)...
    - 0.032077*sin(B) - 0.014615*cos(2*B) - 0.04089*sin(2*B));

% The longitude correction (in minutes) corrects the time from the standard
% meridian (which is the basis for the time zone) to the actual location,
% at 4 minutes per degree longitude.  This is the first case in the if-then
% statement. Occasionally, a site is located across the international date
% line from its standard meridian.  In this case (the 'else' case below)
% the longitudinal angle between the site and its standard meridian is
% calculated as 360 minus the normal difference:
% (360 - (L - M)) = (360 + M - L )

if longitude - meridian < 180
    longcorr = 4*(longitude - meridian);
else
    longcorr = 4*(360 + meridian - longitude);
end

% Solar time (hr) is standard time (hr) plus longitude correction (min) and
% time equation correction (min).
solartime = hour + longcorr/60 + timeeqn/60;

% Hour angle is 15 degrees for each hour away from noon.
hrangle = 15*(solartime - 12);

% Occasionally, the hour angle comes out outside the range of -180 to +180.
% In this case, correct it by adding or subtracting 360.
hrangle(hrangle > 180) = hrangle(hrangle > 180) - 360;
hrangle(hrangle < -180) = hrangle(hrangle < -180) + 360;