function [theta, thetaL, thetaT] = incAngles(~, a, z, Ca, Cs)

% Calculates the three major incidence angles (direct, longitudinal, and
% transverse) on a collector, given the sun's azimuth and zenith, and the
% collector's azimuth orientation and slope.
%
% UNITS
% Inputs and outputs in degrees.
%
% VECTORIZATION
% Configured for vectorized operations: all inputs and outputs can
% be vectors of multiple values.
%
% REFERENCES
% Duffie, 1991: "Solar Engineering of Thermal Processes" - p 14-15
% Theunissen, Beckman, 1985: "Solar transmittance characteristics of evacuated
% tubular collectors with diffuse back collectors"

% Calculate the effective azimuth--the difference between the sun's azimuth
% and that of the collector.
at = a - Ca;

% The main ("direct") angle of incidence is calculated ala Duffie eq 1.6.3.
theta = acosd(cosd(z).*cosd(Cs) + sind(z).*sind(Cs).*cosd(at));

% The direct incidence angle can be decomposed into transverse and
% longitudinal angles, especially important for predicting the performance
% of evacuated tube collectors where the incidence angle modifiers depend
% on the direction of incoming sunlight.

% The longitudinal incidence angle is found by applying Duffie's eq 1.6.3
% to a plane that cuts the collector into "front" and "back" halves.  The
% less sunlight is incident on this plane, the smaller the longitudinal
% incidence angle.  (Whereas in the original equation, the smaller the
% angle, the more incidence--so "acos" changes to "asin".)  This is
% accomplished by adding 90 degrees to the collector slope.
% thetaL = asind(cosd(z)*cosd(Cs+90) + sind(z).*sind(Cs+90).*cosd(at));

% From Theunissen, 1985, longitudinal incidence angle, or the projection of
% the direct incidence angle onto a plane that cuts the collector into
% "front" and "back" halves.
thetaL = atand(tand(theta).*cosd(a));

% The transverse incidence angle is found by applying Duffie's eq 1.6.3
% to a plane that cuts the collector into "left" and "right" halves.  The
% less sunlight is incident on this plane, the smaller the transverse
% incidence angle.  (Whereas in the original equation, the smaller the
% angle, the more incidence--so "acos" changes to "asin".)  This is
% accomplished by replacing the collector slope with 90 degrees, and
% adding 90 degrees to the effective azimuth.
% thetaT = asind(cosd(z)*cosd(90) + sind(z).*sind(90).*cosd(at+90));

% From Theunissen, 1985, transverse incidence angle, or the projection of
% the direct incidence angle onto a plane that cuts the collector into
% "left" and "right" halves.
thetaT = atand(tand(theta).*sind(a));

% Check that this difference equation is zero for all theta < 90.
% diff = theta-atand((tand(thetaL).^2+tand(thetaT).^2).^0.5);

% these checks correct for non-zero outputs due to rounding errors in
% Matlab when solar azimuth and collector azimuth are exactly equal (e.g.
% all the time with 1-d tracking), and when solar zenith and collector tilt
% are also equal (e.g. all the time with 2-d tracking).
theta((a==Ca)&(z==Cs))=0;
thetaT((a==Ca)&(z==Cs))=0;
thetaL((a==Ca)&(z==Cs))=0;

if ~isreal(theta) || ~isreal(thetaL) || ~isreal(thetaT)
    error('imaginary incidence angle calculated');
end

