function [totalradiation, Ib, Id, Ig] = irradianceCalc(~, sys, directN, globalH, diffuseH, etrN, groundrefl, thetaB, thetaBl, thetaBt, z, beta)

% Calculates the total radiation reaching a collector on the basis of five
% sets of data: a system, modeled TMY radiation, calculated incidence
% angles (theta), sun zenith angles (z), and collector slope (beta).
%
% UNITS
% Radiation inputs and outputs in watt-hours per square meter; angle inputs
% in degrees.
%
% VECTORIZATION
% Configured for vectorized operations: all inputs and outputs can
% be vectors of multiple values.
%
% REFERENCES
% Duffie, 1991: "Solar Engineering of Thermal Processes" - p 96-98
% Loutzenhiser et al, 2007: 4381	
% This is commonly known as the Reindl model of solar irradiance and is
% referred to as such in the Loutzenhiser article.
%
% CHECKS
% 1) Global horizontal radiation should equal diffuse horizontal radiation
% plus direct normal radiation times the cosine of the zenith angle.  This
% should be checked for the TMY data set being used.

dataSize = size(globalH);

% Clean up any irradiance data less than zero, as that is impossible.
directN(directN < 0) = 0;
globalH(globalH < 0) = 0;
diffuseH(diffuseH < 0) = 0;
etrN(etrN < 0) = 0;

% The anisotropy index (Ai) "determines the portion of the horizontal diffuse
% radiation which is to be treated as forward scattered" and considered
% incident at the same angle as beam radiation.  Duffie p 97 eq 2.16.3.
% Avoid dividing by zero.
indetr = etrN > 0;
Ai = zeros(dataSize);
Ai(indetr) = directN(indetr)./etrN(indetr);

% Ai should never be greater than 1, so remove all erroneous values: These
% must be data errors.
Ai(Ai > 1) = 0;

% The horizontal-to-tilted ratio (Rb) is the beam radiation incident on the
% tilted collector over the beam radiation incident on a horizontal
% surface.  This simplifies to the ratio of the cosines of incidence angle
% and zenith.
% Avoid dividing by zero.
indzt = (z < 90) & (thetaB < 90);
Rb = zeros(dataSize);
Rb(indzt) = cosd(thetaB(indzt))./cosd(z(indzt));

% Now the various radiation components are calculated (using the notation
% developed by Loutzenhiser et al, since Duffie is inconsistent.

% The incident beam radiation (Itb) is simply the direct normal radiation
% times the cosine of the incidence angle.  (Equivalent to, but simpler
% than, converting to horizontal and then using (Rb), as the references
% do.)
% Calculate only relevant data points: when the sun is over the horizon of
% the earth or panel the incident beam radiation must be zero.
Itb = zeros(dataSize);
Itb(indzt) = directN(indzt).*cosd(thetaB(indzt));

% The incident circumsolar radiation is calculated from the diffuse
% horizontal radiation, the anisotropy index, and the horizontal-to-tilted
% ratio Rb.  It is considered to be in the direction of the beam.
Itdcs = zeros(dataSize);
Itdcs(indzt) = diffuseH(indzt).*Ai(indzt).*Rb(indzt);

% Remove erroneous Itdcs components
Itdcs (Itdcs > 900) = 0;

% Only calculate diffuse and ground reflected components if not a
% concentrating collector.
if ~isa(sys.Collector, 'ConcentratingThermalCollector') && ~isa(sys.Collector, 'PVTCollector') && ~isa(sys.Collector, 'CPVCollector')
    
    % The isotropic sky component of diffuse radiation is calculated using the
    % collector slope to determine how much of the sky is visible, as well as
    % the anisotropy index to determine what portion of diffuse radiation is
    % not circumsolar.
    Itdiso = diffuseH.*(1-Ai).*((1 + cosd(beta))./2);
    
    % The modulating factor (f) accounts for cloudiness.  Duffie eq 2.16.6,
    % implicit in Loutzenhiser eq 8.
    % Avoid dividing by zero.
    indg = globalH ~= 0;
    f = zeros(dataSize);
    f(indg) = sqrt(Itb(indg)./globalH(indg));
    
    % The horizon brightening component of diffuse radiation is calculated in
    % much the same way, with the addition of the modulating factor (f).
    Itdhb = Itdiso.*f.*sind(beta./2).^3;
    
    % The ground component of diffuse radiation is calculated using the
    % collector slope to determine how much of the ground is visible, as well
    % as the ground's diffuse reflectance.
    Itdg = globalH.*groundrefl.*((1 - cosd(beta))./2);
end

% Only calculate IAMs if not a
% 2-axis tracking concentrating collector.
if ~(isa(sys.Collector, 'ConcentratingThermalCollector') && (sys.TrackingAxes == 2)) && ~(isa(sys.Collector, 'PVTCollector') && (sys.TrackingAxes == 2)) && ~(isa(sys.Collector, 'CPVCollector') && (sys.TrackingAxes == 2))
    % Calculate IAM
    
    % First the effective angles of incidence for diffuse and ground reflected
    % components must be determined.  From Duffie these are a function of tilt.
    thetaD = 59.7 - 0.1388.*beta + 0.001497.*beta.^2;
    thetaG = 90 - 0.5788.*beta + 0.002693.*beta.^2;
    
    switch class(sys)
        case 'PVSystem'
            % A physical model for IAM is used for PV
            % Constants from "Physical Model of IAM"  pvpmc.org
            nGlass = 1.526;  % index of refraction of glass
            nAir = 1; % index of refraction of air
            K = 4;      % glazing coefficient (m^-1)
            L = 0.002;  % glazing thickness (m)
            
            n = nGlass/nAir;
            
            IAMb = pvl_physicaliam(K, L, n, thetaB);
            IAMd = pvl_physicaliam(K, L, n, thetaD);
            IAMg = pvl_physicaliam(K, L, n, thetaG);
            
            %             % Calculate angle of refraction (using Snell's law) for the beam, diffuse
            %             % and ground reflection component.
            %             thetaRb = asind(nAir./nGlass.*sind(thetaB));
            %             thetaRd = asind(nAir./nGlass.*sind(thetaD));
            %             thetaRg = asind(nAir./nGlass.*sind(thetaG));
            %
            %             % Calculate transmittance at incident angles of zero and relevant incident
            %             % angle for each component (beam, diffuse, ground reflection).  The ratio
            %             % of these two transmittances is the IAM of each component.
            %             % Transmittance equation is undefined for theta equals zero, so choose a
            %             % theta of 0.1 for stable calculation (pvpmc.org)
            %             theta0 = 0.1;
            %             thetaR0 = asind(nAir./nGlass.*sind(theta0));
            %             tau0 = exp(-K.*L./cosd(thetaR0)).*(1-0.5.*(sind(thetaR0-theta0).^2./sind(thetaR0+theta0).^2 + tand(thetaR0-theta0).^2./tand(thetaR0+theta0).^2));
            %
            %             % Avoid divide by zero errors: when theta is zero, transmissivity should
            %             % be 1
            %             tauB = ones(dataSize);
            %             indt = thetaB ~= 0;
            %             tauB(indt) = exp(-K.*L./cosd(thetaRb(indt))).*(1-0.5.*(sind(thetaRb(indt)-thetaB(indt)).^2./sind(thetaRb(indt)+thetaB(indt)).^2 + tand(thetaRb(indt)-thetaB(indt)).^2./tand(thetaRb(indt)+thetaB(indt)).^2));
            %             tauD = exp(-K.*L./cosd(thetaRd)).*(1-0.5.*(sind(thetaRd-thetaD).^2./sind(thetaRd+thetaD).^2 + tand(thetaRd-thetaD).^2./tand(thetaRd+thetaD).^2));
            %             tauG = exp(-K.*L./cosd(thetaRg)).*(1-0.5.*(sind(thetaRg-thetaG).^2./sind(thetaRg+thetaG).^2 + tand(thetaRg-thetaG).^2./tand(thetaRg+thetaG).^2));
            %
            %             IAMb = tauB ./ tau0;
            %             IAMd = tauD ./ tau0;
            %             IAMg = tauG ./ tau0;
            
        case {'CHPSystem', 'ThermalSystem', 'PVTSystem'}
            % Use a shape preserving piecewise cubic interpolation for calculating
            % the IAM.  This calculation uses an array of IAMs from the EN 12975
            % certificate for the selected Collector.
            
            % The range of incidence angles shown on EN 12975 certificates,
            % plus zero and 90 for completeness.  IAM at zero must be 1, and at
            % 90 must be zero.
            X = [0 10 20 30 40 50 60 70 90];
            
            if isa(sys.Collector, 'TubeCollector')
                
                % For tubular collectors, the longitudinal and transverse
                % IAMs are calculated from interpolations of the EN 12975
                % certificate numbers.  The product of these two is a good
                % approximation of the total beam incidence angle modifier.
                %             IAMbL = zeros(dataSize);
                %             IAMbT = zeros(dataSize);
                IAMbL = interp1(X, sys.Collector.IAMlValues, abs(thetaBl));
                IAMbT = interp1(X, sys.Collector.IAMtValues, abs(thetaBt));
                IAMb = IAMbL .* IAMbT;
                
                % From Theunissen, 1985, good values for the longitudinal and
                % transverse incidence angles for diffuse and ground reflected
                % radiation are given as 57 and 37 degrees, respectively.
                thetaDl = 57;
                thetaDt = 37;
                IAMdL = interp1(X, sys.Collector.IAMlValues, thetaDl);
                IAMdT = interp1(X, sys.Collector.IAMtValues, thetaDt);
                IAMd = IAMdL .* IAMdT;
                IAMg = IAMd;
                
            elseif (isa(sys.Collector, 'ConcentratingThermalCollector') || isa(sys.Collector, 'PVTCollector')) && (sys.TrackingAxes == 1)
                % for North-South 1-axis tracking concentrating collectors only the
                % transverse IAM matters.
                IAMb = zeros(dataSize);
                IAMb(thetaB<=90) = interp1(X, sys.Collector.IAMtValues, thetaB(thetaB<=90));
                % Diffuse and ground reflected related parameters are all
                % zero.
                IAMd = zeros(dataSize);
                IAMg = zeros(dataSize);
                Itdiso = zeros(dataSize);
                Itdhb = zeros(dataSize);
                Itdg = zeros(dataSize);
                
            else
                % for a standard non-concentrating flat plate collector
                IAMb = zeros(dataSize);
                IAMb(thetaB<=90) = interp1(X, sys.Collector.IAMlValues, thetaB(thetaB<=90));
                IAMd = interp1(X, sys.Collector.IAMlValues, thetaD);
                IAMg = interp1(X, sys.Collector.IAMlValues, thetaG);
                
            end
        otherwise
            error('Invalid system type: please specify a "ThermalSystem", "CHPSystem", "PVSystem, or "PVTSystem"')
    end
else
    % For 2-axis tracking concentrating collectors the diffuse and ground reflected
    % components are set to zero, IAMb is always one, and IAMd and IAMg are
    % zero.
    IAMb = ones(dataSize);
    IAMd = zeros(dataSize);
    IAMg = zeros(dataSize);
    Itdiso = zeros(dataSize);
    Itdhb = zeros(dataSize);
    Itdg = zeros(dataSize);
end

% The total radiation on the collector is the sum of five components:
% beam, diffuse circumsolar, diffuse isotropic sky, diffuse horizon
% brightening, and diffuse ground, multiplied by their respective Incidence Angle
% Modifiers.  Incidence angle modifiers are grouped into 3
% categories: beam, diffuse, and ground reflected.

Ib = IAMb.*(Itb + Itdcs);
Id = IAMd.*(Itdiso + Itdhb);
Ig = IAMg.*Itdg;

totalradiation = Ib + Id + Ig;

% Error checking
if any(totalradiation > 2000)
    error('Data error:  calculated total radiation is greater than 2000 W/m^2')
end
if any(Itb < 0) || any(Itdcs < 0)|| any(Itdiso < 0) || any(Itdhb < 0) || any(Itdg < 0)
    error('Data error:  Some component of calculated irradiance is less than zero')
end
end

