function [azmth, znth] = sunangles(~, latitude, declination, hourangle)

% Calculates the sun's azimuth and zenith given the location's latitude,
% the declination (a function of date), and the hour angle (a function of
% time).
%
% UNITS
% Inputs and outputs in degrees.
%
% VECTORIZATION
% Configured for vectorized operations: all inputs and outputs can
% be vectors of multiple values.
%
% REFERENCES
% Duffie, 1991: "Solar Engineering of Thermal Processes" - p 16
% Brauna & Mitchell: "Solar geometry for fixed and tracking surfaces"
% Pepe's writeup: "Solar Angles--Theory and Vectorized Matlab
%   Implementation"

% ANGLE CONVERSIONS

% Convert all inputs from degrees to radians.
lat = deg2rad(latitude);
decl = deg2rad(declination);
hrang = deg2rad(hourangle);

% ZENITH

% Calculate zenith angle (Duffie eq 1.6.5) and convert it to degrees.
z = acos(cos(lat)*cos(decl).*cos(hrang) + sin(lat)*sin(decl));
znth = single(rad2deg(z));

% AZIMUTH

% Calculate pseudo-surface-azimuth angle (Duffie eq 1.6.6c).
azp = asin(sin(hrang).*cos(decl)./sin(z));

% To determine the correct azimuth angle, it is necessary to determine
% which quadrant the sun is in.  This is accomplished with three
% C-coefficients. 
%
% C-coefficients are initialized as vectors as long as the input vectors.
% They are initialized with default values of +1.
% 
% C2 = 1 if latitude*(latitude - declination) > 0
%       -1 otherwise
% C1 = 1 if C2 == 1
%        or latitude*(latitude + declination) > 0
%        or |hourangle| > ewhourangle
%       -1 otherwise
% C3 = 1 if hourangle >= 0
%       -1 otherwise
%
% This represents three changes from Duffie's "Solar Engineering of Thermal
% Processes":
% 1. In C2, latitude multiplier added to agree with original paper.
% 2. In C2, the +1 case is changed from ">=0" to ">0", to make the new
% inequality work at the equator where latitude = 0.  (A problem introduced
% by change #1 above.)
% 3. C1 depends on ewhourangle only if two conditions are met:
%   3a. C2 is +1, because if C2 is -1, ewhourangle cannot be evaluated.
%   3b. latitude*(latitude+declination) > 0, because otherwise, ewhourangle
%   again cannot be eveluated.  In both cases, the result of evaluating
%   ewhourangle is imaginary.
%   In physical terms, C1 is -1 only when the sun is behind the observer,
%   if the observer is facing in the sun's noon-time azimuth direction (due
%   north or due south).

% Create C matrix, filled with the default value of -1.  C1 is stored in
% the first column C(:,1), C2 in C(:,2), and C3 in C(:,3).
C = -1*ones(length(azp),3);

% Change C2 to 1 where appropriate.
latvector = lat*ones(length(azp),1);
lattest1 = latvector.*(latvector - decl);
C(gt(lattest1,0),2) = 1;

% Next, deal with C1:
% Create a vector for east-west hour angle, populated at first with NaN.
ewhrang = NaN*ones(length(azp),1);
% Determine the values of lat*(lat + decl).
lattest2 = latvector.*(latvector + decl);
% Determine where it is possible to calculate the east-west hour angle:
% where C2 == 1 and lattest2 > 0.
ewOK = and(C(:,2)==1, gt(lattest2,0));
% Where possible, calculate the east-west hour angle (Duffie eq 1.6.6g).
ewhrang(ewOK,1) = ...
    acos(tan(decl(ewOK)).* ...
    cot(latvector(ewOK)));
% Finally, change C1 to 1 where appropriate (the east-west hour angle was
% not calculated--or it was, but the absolute value of the hour angle is
% less than it).
C(or(isnan(ewhrang),lt(abs(hrang),ewhrang)),1) = 1;

% Change C3 to -1 where appropriate.
C(ge(hrang,0),3) = 1;

% Finally, the true azimuth is calculated (Duffie eq 1.6.6a) and converted
% to degrees.
az = C(:,1).*C(:,2).*azp + C(:,3).*(1 - C(:,1).*C(:,2))/2*pi;
azmth = single(rad2deg(az));