classdef ThermalCollector < HeatExchanger
    %THERMALCOLLECTOR Thermal collector class
    %   Thermal collectors are defined by the simpler efficiency equation 
    % with three coefficients, and IAMValues at 0, 10, 20, 30, 40, 50, 60,
    % 70 and 90 degrees as defined in SRCC standards and test standard
    % EN 12975.  One set is longitudinal, and one transverse IAMs, and for
    % flat-plate collectors these are identical.
    
    properties
        StateIn
        StateOut
        F
        A1
        A2
        IAMtValues
        IAMlValues
        Wp % Wp per m^2 module area
        Area % module area
    end
    
    methods
        function thermalCollector = ThermalCollector(f, a1, a2, iamLvalues, iamTvalues, wp, area)
            if nargin == 0
                warning('Null input constructor call: ThermalCollector');
            elseif nargin == 7
                thermalCollector.F = f;
                thermalCollector.A1 = a1;
                thermalCollector.A2 = a2;
                thermalCollector.IAMtValues = iamTvalues;
                thermalCollector.IAMlValues = iamLvalues;
                thermalCollector.Wp = wp;
                thermalCollector.Area = area;
                thermalCollector.StateIn = [];
                thermalCollector.StateOut = [];
            else
                error('Not correct number of inputs. Please specify 6 inputs for a thermal collector')
            end
        end
        
        function heatOut = getHeat(collector, stateIn, temperatureOut)
            if nargin ~= 3
                error('Please specify 3 inputs to the ThermalCollector.calculate function')
            end
            collector.StateIn = stateIn;
            collector.StateOut = FluidState(collector.StateIn.FluidModel, 'pressure', collector.StateIn.Pressure,'temperature', temperatureOut);
            heatOut = (collector.StateOut.Enthalpy - collector.StateIn.Enthalpy);
        end
        
        function etaThermal = getEfficiency(thermalCollector, KG, Ta)
            if isempty(thermalCollector.StateIn) || isempty(thermalCollector.StateOut)
                error('Efficiency cannot be calculated until both input and output states have been specified.')
            elseif nargin == 3
                etaThermal = thermalCollector.F - thermalCollector.A1./KG.*((thermalCollector.StateIn.Temperature + thermalCollector.StateOut.Temperature) ./ 2 - Ta) - thermalCollector.A2./KG.*((thermalCollector.StateIn.Temperature + thermalCollector.StateOut.Temperature) ./ 2 - Ta).^2;
                %Remove divide by zero and negative efficiency data
                etaThermal(etaThermal<0)=0;
            else
                error('Please specify 3 inputs to the ThermalCollector.getEfficiency function')
            end
        end
        
        
    end
    
end

