function [] = PlotElinSolarMap( mapELINnames, data, gridSpacing, titleText, cLabel, loLim, hiLim)
%PlotElinSolarMap plots the new installed solar capacity at every solar site.
%   Detailed explanation goes here

Giga = 1e9;

% load data for european countries (STAT_LEVL is 0) from NUTS shapefile from Eurostat
countries = shaperead('NUTS_RG_10M_2006.shp', 'Attributes', {'NUTS_ID'}, 'Selector', {@(v1) v1==0, 'STAT_LEVL_'}, 'UseGeoCoords', true);

% preinitialize array variables
splitStr = cell(1,length(data.uels{2}));
xIndex = zeros(1,length(data.uels{2}));
yIndex = zeros(1,length(data.uels{2}));
quadArea = zeros(1,length(data.uels{2}));

% set the earth ellipsoid with units of meters
earthEllipsoid = referenceEllipsoid('grs80', 'm');

% split the solar site text names into the latitude and longitude parts
% using regular expressions.
for i=1:length(data.uels{2})
    expression = '[NE]';
    splitStr{i} = regexp(data.uels{1,2}{1,i},expression,'split');
    
    % if one of the split strings has an underscore as the first character
    % this means it is negative, so replace it with a negative sign.
    for j=2:3
        if splitStr{i}{1,j}(1) == '_'
            splitStr{i}{1,j}(1) = '-';
        end
    end
    
    % Replace the remaining underscores in the latitude and longitude with
    % dots indicating the decimal point.
    expression = '_';
    splitStr{i} = regexprep(splitStr{i},expression,'.');
    
    % Convert the latitude and longitude strings to numbers and save them
    % in the data structure.
    data.latitude(i) = str2double(splitStr{i}{1,2});
    data.longitude(i) = str2double(splitStr{i}{1,3});
    
end

% Set the appropriate lat/lon limits for the plots based on the data.
latMin = min(data.latitude);
latMax = max(data.latitude);
% latmax = 62.125;
lonMin = min(data.longitude);
% lonmin = -11.125;
lonMax = max(data.longitude);

% Select only the last year and get rid of the extra dimension in the data
% array (squeeze), and convert from GWp to Wp.
dataSize = size(data.val);
data.val = data.val(:,:,dataSize(3))'.*Giga;

% First find the size of the array assuming a lat/long grid with spacing specified
% from the min to max latitude/longitude.
xLength = length(latMin:gridSpacing:latMax);
yLength = length(lonMin:gridSpacing:lonMax);

for i=1:length(data.uels{2})
    % Convert the latitude and longitude fields for each data point
    % into indices of the array.
    xIndex(i) = (data.latitude(i) - latMin)*(1/gridSpacing) + 1;
    yIndex(i) = (data.longitude(i) - lonMin)*(1/gridSpacing) + 1;
    % Calculate the area of the quadrangle
    [latGeoQuad, lonGeoQuad] = outlinegeoquad([data.latitude(i) - gridSpacing/2, data.latitude(i) + gridSpacing/2], [data.longitude(i) - gridSpacing/2, data.longitude(i) + gridSpacing/2], gridSpacing, gridSpacing);
    quadArea(i) = areaint(latGeoQuad, lonGeoQuad, earthEllipsoid);
end

% Create a regular grid of the data so that it can be plotted as a
% contour plot.
for j=1:length(data.uels{1})
    % Initialize the data grid to zeros.
    dataGrid = zeros(xLength,yLength);
    
    % Add the data to the array
    for i=1:length(data.uels{2})
        dataGrid(xIndex(i),yIndex(i)) = data.val(i,j) / quadArea(i);
    end
    
    % check if dataGrid has any nonzero values and only produce a graph for
    % that condition
    if any(dataGrid(:) ~= 0)
        
        dataGrid(dataGrid == 0) = NaN;
        
        figTitle = [mapELINnames(data.uels{1}{j}), ' -- ', titleText];
        
        % Create a figure for each solar technology
        fig = figure('NumberTitle','off',...
            'Name', figTitle);
        axes2 = axesm('MapProjection', 'eqaconicstd',...
            'MapLatLimit',[latMin latMax],'MapLonLimit', [lonMin lonMax],...
            'Frame','on');

        % show the countries, colored white
        geoshow(axes2, countries, 'FaceColor', [1 1 1]);

        % Create surface plot
        set(axes2,'Position',[0.01 0.01 0.98 0.98]);
        surfm([latMin latMax], [lonMin lonMax], dataGrid, 'Parent',axes2);
        
        % Create colorbar
        % If the lower color limit is set at -1 (autoscale), set it as the lowest
        % value found in any of the data.  Otherwise set it to the specified value.
        if loLim == -1
            cMin = min(min(dataGrid(~isnan(dataGrid))));
        else
            cMin = loLim;
        end
        
        % If the upper color limit is set at -1 (autoscale), set it as the highest
        % value found in any of the data.  Otherwise set it to the specified value.
        if hiLim == -1
            cMax = max(max(dataGrid(~isnan(dataGrid))));
        else
            cMax = hiLim;
        end
        
        % if there is only one value then cMin will equal cMax, but to get
        % a valid color scale set cMin to zero in this case.
        if cMin == cMax
            cMin = 0;
        end
        
        caxis(axes2, [cMin, cMax]);
        cbar2 = colorbar('peer',axes2);
        set(cbar2,'Position',[0.87 0.03 0.04 0.9]);
        colormap(linspecer);
        ylabel(cbar2, cLabel);
        tightmap
        
        % Save the figures to various image formats
        saveas(fig,[figTitle '.eps'],'epsc');
        saveas(fig,[figTitle '.png']);
        saveas(fig,[figTitle '.fig']);
    end
end

end

