function [] = PlotElinWindMap( data, latMin, latMax, lonMin, lonMax, titleText, cLabel, loLim, hiLim)
%PlotElinWindMap plots the new installed wind capacity density in every nuts2 region.
%   Detailed explanation goes here

Giga = 1e9;
Mega = 1e6;

% load data for european nuts2 regions (STAT_LEVL is 2) from NUTS shapefile from Eurostat
nuts2regions = shaperead('NUTS_RG_10M_2006.shp', 'Attributes', {'NUTS_ID'}, 'Selector', {@(v1) v1==2, 'STAT_LEVL_'}, 'UseGeoCoords', true);



% Sum all the onshore wind classes into one column,
data.val = [sum(data.val(:,1:8,:),2), data.val(:,9,:)];
% take only the last year of data and covert to Wp from GWp.
dataSize = size(data.val);
data.val = data.val(:,:,dataSize(3)).*Giga;
% and update the uels and dataSize for the resulting data.
data.uels = {data.uels{1}, {'onshore','offshore'}};
dataSize = size(data.val);

% set the earth ellipsoid with units of meters
earthEllipsoid = referenceEllipsoid('grs80', 'm');

% Calculate the areas of each nuts2 region and add this to the nuts2regions
% structure.
for k=1:length(nuts2regions)
    nuts2regions(k).Area = sum(areaint(nuts2regions(k).Lat, nuts2regions(k).Lon, earthEllipsoid));
end

% read total land area data per nuts 2 region from Lenas Area_NUTS2.xlsx
% file.  The data is divided into two worksheets, so concatenate them in
% the final result and convert to square meters from kilometers.
% [nuts2regions.Area, nuts2regions.NUTS_ID, ~] = xlsread('Area_NUTS2.xlsx','tot_area_1');
% [nuts2regions2.Area, nuts2regions2.NUTS_ID, ~] = xlsread('Area_NUTS2.xlsx','tot_area_2');
[area1, nuts_id1, ~] = xlsread('Area_NUTS2.xlsx','tot_area_1');
[area2, nuts_id2, ~] = xlsread('Area_NUTS2.xlsx','tot_area_2');
areaELIN = [area1, area2].*Mega;
nuts_idELIN = [nuts_id1, nuts_id2];

% for k=1:length(area1)
%     nuts2regionsLena(k).Area = area1(k).*Mega;
%     nuts2regionsLena(k).NUTS_ID = nuts_id1{k};
% end
% for l=1:length(area2)
%     k=k+1;
%     nuts2regionsLena(k).Area = area2(l).*Mega;
%     nuts2regionsLena(k).NUTS_ID = nuts_id2{l};
% end

% read total land area data per nuts 2 region from eurostat .tsv file
% areaFile = tdfread('tgs00002.tsv');
% alternate area from eurostat data
for k=1:length(nuts2regions)
    %     index = find(strcmp(cellstr(areaFile.unit0x2Clanduse0x2Cgeo0x5Ctime(:,11:14)),nuts2regions(k).NUTS_ID));
    index = find(strcmp(nuts_idELIN, nuts2regions(k).NUTS_ID));
    if any(index)
        %          if areaFile.x2006(index(1))~=':'
        % take year 2006 to match the shapefile year
        %             nuts2regions(k).AreaELIN = str2double(cellstr(areaFile.x2006(index(1),:))) * Mega;
        nuts2regions(k).AreaELIN = areaELIN(index);
        %             nuts2regions(k).Area = nuts2regions(k).Area2;
        %             nuts2regions(k).AreaError = (nuts2regions(k).Area2 - nuts2regions(k).Area)/nuts2regions(k).Area2;
        nuts2regions(k).AreaError = (nuts2regions(k).AreaELIN - nuts2regions(k).Area)/nuts2regions(k).Area;
        %         end
    else
        %         warning('no Eurostat land area for that NUTS2 region');
        warning('no ELIN land area for that NUTS2 region');
    end
end

% Add the capacity density data to the existing regions structure and create
% figures to output the results
for j=1:dataSize(2)
    % Create a new capacity density field in the nuts2regions structure and initialize it to zero.
    for k=1:length(nuts2regions)
        nuts2regions(k).CapacityDensity = 0;
    end
    
    % Calculate the installed capacity density (W/m^2) for each region in
    % 2050 using ELIN region areas not the shapefile areas to be consistent
    % with the data used in ELIN for available wind installation area.
    for i=1:dataSize(1)
        index = find(strcmp({nuts2regions.NUTS_ID},data.uels{1}(i)));
        if any(index)
            if (data.val(i,j)>0)
                nuts2regions(index).CapacityDensity = data.val(i,j) / nuts2regions(index).AreaELIN;
            else
                nuts2regions(index).CapacityDensity = NaN;
            end
        elseif (data.val(i,j)>0)
            warning('wind capacity installed in non nuts2 region');
        end
    end
    
    % check if there are any numeric values (non-zero) and only produce a figure for
    % that condition
    if any(~isnan([nuts2regions.CapacityDensity]))
        
        % Create a figure for each technology
        figTitle = [data.uels{2}{j}, ' -- ', titleText];
        fig = figure('NumberTitle','off',...
            'Name', figTitle);
        axes = axesm('MapProjection', 'eqaconicstd',...
            'MapLatLimit',[latMin latMax],'MapLonLimit', [lonMin lonMax],...
            'Frame','on');
        set(axes,'Position',[0.01 0.01 0.98 0.98]);
        
        % create colormap
        
        % If the lower color limit is set at -1 (autoscale), set it as the lowest
        % value found in any of the data.  Otherwise set it to the specified value.
        if loLim == -1
            cMin = min([nuts2regions.CapacityDensity]);
        else
            cMin = loLim;
        end
        
        % If the upper color limit is set at -1 (autoscale), set it as the highest
        % value found in any of the data.  Otherwise set it to the specified value.
        if hiLim == -1
            cMax = max([nuts2regions.CapacityDensity]);
        else
            cMax = hiLim;
        end
        
        % if there is only one value then cMin will equal cMax, but to get
        % a valid color scale set cMin to zero in this case.
        if cMin == cMax
            cMin = 0;
        end
        
        colormap(linspecer);
        caxis(axes, [cMin cMax]);
        
        % Create a mapping of each nuts2regions capacity density value to the colormap
        % chosen and plot it on the axes.  Setting the default FaceColor to
        % white will make the regions with no installed capacity (NaN) white [1 1 1].
        dataSymbolSpec = makesymbolspec('Polygon', {'CapacityDensity',[cMin, cMax],'FaceColor',colormap}, {'Default', 'FaceColor', [1 1 1]});
        % show the countries, colored white
        geoshow(axes, nuts2regions, 'FaceColor', [1 1 1]);
        
        geoshow(axes, nuts2regions, 'SymbolSpec', dataSymbolSpec);
        
        % Create colorbar
        cbar2 = colorbar('peer',axes);
        set(cbar2,'Position',[0.87 0.03 0.04 0.9]);
        ylabel(cbar2, cLabel);
        tightmap
        
        % Save the figures to various image formats
        saveas(fig,[figTitle '.eps'],'epsc');
        saveas(fig,[figTitle '.png']);
        saveas(fig,[figTitle '.fig']);
    end
end

end



